#!/usr/bin/env python
#
# Generates Makefile for IMACS and LDSS3 data reduction
# using Gus's programs based on Cosmos2 and our programs.
#
# Need to added noise and gain parameters for LDSS3 data
#
# Build newer version with classes
#
# Last modified 10-20-06

import glob,sys,os,re,string,shutil,datetime,popen2
from timestamp import timestamp 
from Numeric import *

##################################################################################################

FLATLIST = ["flat","incan","quartz"]
LAMPLIST = ["lamp","arc","henear","he ne ar","he-ne-ar","he+ne+ar","th-ar"]
BLACKLIST = FLATLIST + LAMPLIST + ["align","bias","mask","direct","image"]

### the difference between imacs and imacs2 is the site and site2 (newer replacement ccd)

### Obsdef files
instruments = {"imacs":{"GR_ANGLE":0,\
                        "GRATING":["IMACS_grism_150","IMACS_grism_200","IMACS_grism_300",\
                                   "IMACS_grism_600","IMACS_300A","IMACS_600A","IMACS_1200A",
                                   "MOE"],\
                        "INSTRUMENT":"IMACS",\
                        "GR_ORDER":1,\
                        "DEWAR":"SITE",\
                        "CAMERA":["SHORT","LONG"],\
                        "MODE":"SPEC"},\
               "imacs2":{"GR_ANGLE":0,\
                         "GRATING":["IMACS_grism_150","IMACS_grism_200","IMACS_grism_300",\
                                    "IMACS_grism_600","IMACS_300A","IMACS_600A","IMACS_1200A",
                                    "MOE"],\
                         "INSTRUMENT":"IMACS",\
                         "GR_ORDER":1,\
                         "DEWAR":"SITE2",\
                         "CAMERA":["SHORT","LONG"],\
                         "MODE":"SPEC"},\
               "ldss2":{"GR_ANGLE":0,\
                        "GRATING":["LDSS2_low","LDSS2_med_blue","LDSS2_med_red"],\
                        "INSTRUMENT":"LDSS2",\
                        "GR_ORDER":1,\
                        "DEWAR":"LDSS3",\
                        "CAMERA":None,\
                        "MODE":"SPEC"},\
               "ldss3":{"GR_ANGLE":0,\
                        "GRATING":["LDSS3_med_red","LDSS3_VPHred","LDSS3_VPHblue",\
                                   "LDSS3_high_blue","LDSS3_high_red"],\
                        "INSTRUMENT":"LDSS3",\
                        "GR_ORDER":1,\
                        "DEWAR":"LDSS3",\
                        "CAMERA":None,\
                        "MODE":"SPEC"}}

### Dewar Offset files
# IMACS Short Camera
SC200g_N = """#
#CCD Dewar offset file based on SC direct, Normal orientation, 200l grism template
#
179.957 1.00132   -0.743   -0.128"""

SC300g_N = """#
#CCD Dewar offset file based on SC direct, Normal orientation, 300l grism template
#
179.929 1.00088   -0.659   -0.060"""

SC150g_NS = """#
#CCD Dewar offset file based on SC direct, N&S orientation, 200l grism template
#
 90.033 1.00073   -0.595    0.234 1"""

SC200g_NS = """#
#CCD Dewar offset file based on SC direct, N&S orientation, 200l grism template
#
 89.957 1.00132   -0.743   -0.128"""
 
SC300g_NS = """#
#CCD Dewar offset file based on SC direct, N&S orientation, 300l grism template
#
 89.929 1.00088   -0.659   -0.060"""

SC600g_NS = """#
#CCD Dewar offset file for SC, N&S orientation, 600l grism template
#
 90.072 0.99983   -1.022    0.157"""

# IMACS Long Camera
LC600_5l_N = """#
#CCD Dewar offset file for Long Camera + 600l 5000A grating, Normal orientation
#
 180.202 0.99863   -1.130   -0.366"""

LC300l_NS = """#
#CCD Dewar offset file for Long Camera + 300l grating, Nod&Shuffle orientation
#
 90.160 0.99821    0.939   -2.004"""

LC600_5l_NS = """#
#CCD Dewar offset file for Long Camera + 600l 5000A grating, Nod&Shuffle orientation
#
 90.202 0.99863   -1.130   -0.366"""

MOE = """#
#CCD Dewar offset file for Long Camera direct images, Nod&Shuffle orientation
#
 90.290 1.00291   -5.433    8.875 1"""

# LDSS3
L3med_R = """#
#CCD Dewar offset file for LDSS3 with old medium red grism
#
-90.594 1.00315    0.162   -1.195 1"""

L3VPH_R = """#
#CCD Dewar offset file for SC direct, Nod & Shuffle orientation
#
 -90.762 1.00461   0.640  -1.095 1"""

L3VPH_B = """#
#CCD Dewar offset file for SC direct, Nod & Shuffle orientation
#
-90.535 1.00272   -0.222   -1.566 -1"""


dewarobs = {# IMACS Short Camera
            "Grism1_200l":{"SC200g_N":SC200g_N,"SC200g_NS":SC200g_NS},\
            "Grism2_300l":{"SC300g_N":SC300g_N,"SC300g_NS":SC300g_NS},\
            "Grism3_300l":{"SC300g_N":SC300g_N,"SC300g_NS":SC300g_NS},\
            "Gri-150-18.8":{"SC150g_NS":SC150g_NS},\
            "Gri-200-15.0":{"SC200g_NS":SC200g_NS},\
            "Gri-300-17.5":{"SC300g_NS":SC300g_NS},\
            "Gri-600-34.0":{"SC600g_NS":SC600g_NS},\
            # IMACS Long Camera
            "300_4.3":{"LC300l_NS":LC300l_NS},\
            "Gra-300-4.3":{"LC300l_NS":LC300l_NS},\
            "Gra-600-8.6":{"LC600.5l_NS":LC600_5l_NS},\
            "Gra-600-13.0":{"LC600.5l_NS":LC600_5l_NS},\
            "Gra-1200-17.5":{"LC600.5l_NS":LC600_5l_NS},\
            "Gra-1200-26.7":{"LC600.5l_NS":LC600_5l_NS},\
            "MOE":{"MOE":MOE},\
            # LDSS3
            "Old_Med_Red":{"L3med_R":L3med_R},\
            "VPH_Red":{"L3VPH_R":L3VPH_R},\
            "VPH_Blue":{"L3VPH_B":L3VPH_B}}

### Dispersion

dispersion = {# IMACS Short Camera
              "Grism1_200l":2.037,\
              "Grism2_300l":1.341,\
              "Grism3_300l":1.341,\
              "Gri-150-18.8":2.700,\
              "Gri-200-15.0":2.037,\
              "Gri-300-17.5":1.341,\
              "Gri-600-34.0":0.583,\
              # IMACS Long Camera
              "300_4.3":0.743,\
              "Gra-300-4.3":0.743,\
              "Gra-600-8.6":0.378,\
              "Gra-600-13.0":0.387,\
              "Gra-1200-17.5":0.194,\
              "Gra-1200-26.7":0.188,\
              "MOE":1.0,\
              # LDSS3
              "Old_Med_Red":2.26,\
              "VPH_Red":1.175,\
              "VPH_Blue":0.682}

argblacklist = ("-1-0","--cosmos1-0","-2-0","--cosmos2-0","-2-02","--cosmos2-02","-2-04",\
                "--cosmos2-04","-cl","--comlines","-db","--database","-e1","--exclude1",\
                "-e2","--exclude2","-f","--flats","-ins","--instrument","-l","--list","-mk",\
                "--make","-mb1","--medbox1","-mb2","--medbox2","-ns1","--no2dspline1","-ns2",\
                "--no2dspline2","-nb","--nobak","-ns","--nosplit","-o","--optimal","-od",\
                "--ord-disp","-os","--ord-sag","-pd","--pdict","-rm","--remake","-rr","--rerun",\
                "-sf","--skyfit","-sh","--search-height","-sl","--skylines","-sl1","--siglimit1",\
                "-sl2","--siglimit2","-slw","--slit-width","-sw","--search-width","-h","--help") 
                # ommitting -t and --targets

##################################################################################################

def help():
    print
    print "-1-0  --cosmos1-0                     Cosmos 1.0 parameter files"
    print "-2-0  --cosmos2-0                     Cosmos 2.0 parameter files"
    print "-2-02 --cosmos2-02                    Cosmos 2.02 parameter files"
    print "-2-04 --cosmos2-04                    Cosmos 2.04 parameter files"
    print "-cl   --comlines      LINEFILE        comparison linelist (default is afea.dat)"
    print "-db   --database      DATABASEFILE    database file"
    print "-e1                   EXCLUDENUM      exclude1"
    print "-e2                   EXCLUDENUM      exclude2"
    print "-l    --list                          lists objects in db file"
    print "-h    --help                          print program options"
    print "-ins  --instrument    INSTRUMENT      choose the instrument you want the pipeline for [imacs,imacs2,ldss2,ldss3]"
    print "-f    --flats                         use muliple flats (LDSS3)"
    print "-mk   --make          MAKEFILENAME    name of makefile"
    print "-mb1                  MEDNUM          medbox1"
    print "-mb2                  MEDNUM          medbox2"
    print "-nb   --nobak                         no backups made of parameter files"
    print "-ns   --nosplit                       no spliting of HOLES and SLITS for second pass"
    print "-ns1  --no2dspline1                   no spline2d1"
    print "-ns2  --no2dspline2                   no spline2d2"
    print "-o    --optimal                       sets optimal subsky for first pass"
    print "-od   --ord-disp      ORDNUM          ord-disp"
    print "-os   --ord-sag       ORDNUM          ord-sag"
    print "-pd   --pdict                         print target dictionary"
    print "-rm   --remake                        rebuilds makefile w/o editing data files"
    print "-rr   --rerun                         rebuilds makefile in non-interactive mode w/o editing data files"
    print "-sf   --skyfit                        sets imacsSkyFit as the sky-subtraction routine (default is subsky)"
    print "-t    --targets       TARG1 TARG2 ... name of targets to reduce"
    print "-sh   --search-height SEARCHNUM       search_height"
    print "-sl   --skylines      LINELIST        skylines linelist (default is HeNeAr.dat)"
    print "-sl1  --siglimit1     SIGNUM          siglimit1"
    print "-sl2  --siglimit2     SIGNUM          siglimit2"
    print "-slw  --slit-width    SLITNUM         slit_width"
    print "-sw   --search-width  SEARCHNUM       search_width"
    print "-wr   --wrange        WMIN - WMAX     wavelength range for extraction (default is 4000-9500)"
    print
    print "e.g." 
    print "CosmosPipeline.py -db ut20040511.db -ins imacs"
    print

def reduce(directory,flats,sciences,lamps,dewfile,makefile,setup,optimal,wrange,skylines,comlines,dp,nobak,skyfit,cosmos,instrument,rerun,nfits,smf,split,Hinstrument,mflats,version,testbuild,longcamera,ddegree,increment,binning,extractlamp):
   
    nsciences = [os.path.basename(science) for science in sciences]
    iflats = [os.path.basename(flat) for flat in flats]
    ilamps = [os.path.basename(lamp) for lamp in lamps]

    jsciences= array([int(science.replace("ccd","")) for  science in nsciences])
    #   jXXXXX --> array containing "ccd" file numbers
    jflats = array([int(flat.replace("ccd","")) for flat in iflats])

    if mflats == 1:
        #   uXXXXX --> list containing relevant indices for the flats associated with each science, may contain duplicates
        uflats = [argmin(abs(j-jflats)) for j in jsciences]
        #   nXXXXX --> uXXXXX with redudancies eliminated
        nflats = []; [nflats.append(j) for j in uflats if j not in nflats]
    elif mflats == 2:
        ### Need a condition for uneven flats (i.e. no flats at the end or the beginning), switch to single flat mode for those objects.
        cflats = []
        for j in jsciences:
            Down = []
            Up = []
            for k in j-jflats:
                if k < 0: Up.append(k)
                else: Up.append(-1000)
                if k > 0: Down.append(k)
                else: Down.append(1000)
            cflats.append([argmin(array(Down)),argmax(array(Up))])
        nflats = []; [nflats.append(j) for j in concatenate(cflats) if j not in nflats]
        gflats = []; [gflats.append(j) for j in cflats if j not in gflats]

    jlamps = array([int(lamp.replace("ccd","")) for lamp in ilamps])
    ulamps = [argmin(abs(j-jlamps)) for j in jsciences]
    nlamps = []; [nlamps.append(j) for j in ulamps if j not in nlamps]

    flist = [iflats[n] for n in nflats]
    llist = [ilamps[n] for n in nlamps]
    
    print "sciences=",nsciences
    print "flats=",flist
    print "lamps=",llist

    f = open(makefile,"w")


    f.write("# Cosmos Pipeline version %s\n\n" % (version))
    f.write("# mask %s\n" % (directory))
    f.write("# created %s\n\n" % (timestamp()))
    f.write("# sciences=%s\n# flats=%s\n# lamps=%s\n\n" % (nsciences,flist,llist))

    if Hinstrument in ("LDSS3-Two","LDSS3-Four"):
        stage1 = "stage-par stage-stitch stage-align-mask stage-map-spectra stage-adjust-map"
        nfits = 2
    elif longcamera:
        stage1 = "stage-par stage-long-camera stage-align-mask stage-map-spectra stage-adjust-map"
    else:
        stage1 = "stage-par stage-align-mask stage-map-spectra stage-adjust-map"
        
    stage2 = "stage-sflats stage-biasflat stage-subsky stage-extract-2dspec"
    if testbuild: 
        if len(nsciences) > 1: stage2 += " stage-sumspec"
        stage_mosaic  = "stage-mosaic"
        stage_perl  = "stage-perl"
        stage_iraf  = "stage-iraf"
        
    if extractlamp: stage3 = "stage-par-2 stage-2pass stage-map-spectra-2 stage-adjust-map-2 stage-subsky-2 stage-extract-2dspec-2 stage-sumspec stage-extract-flat stage-extract-lamp stage-extract-1dspec"
    else: stage3 = "stage-par-2 stage-2pass stage-map-spectra-2 stage-adjust-map-2 stage-subsky-2 stage-extract-2dspec-2 stage-sumspec stage-extract-flat stage-extract-1dspec"
    stage4 = "stage-flux"
    stage5 = "stage-redshift"
    stage6 = "stage-redshift-output"


    #all = " ".join([stage1,stage2,stage3,stage4,stage5])
    if testbuild: all = " ".join([stage1,stage2,stage_mosaic,stage_perl,stage_iraf])
    else: all = " ".join([stage1,stage2,stage3,stage4,stage5,stage6])

    f.write("all: %s\n\n" % (all))
    f.write("stage1: %s\n\n" % (stage1))
    f.write("stage2: %s\n\n" % (stage2))
    if testbuild:
        '''Nothing here yet'''
        #f.write("stage-misc: %s\n\n" % (stage_misc))
        #f.write("stage-perl: %s\n\n" % (stage_perl))
        #f.write("stage-iraf: %s\n\n" % (stage_iraf))
    else:
        f.write("stage3: %s\n\n" % (stage3))
        f.write("stage4: %s\n\n" % (stage4))
        f.write("stage5: %s\n\n" % (stage5))
        f.write("stage6: %s\n\n" % (stage6))

    f.write("include parameters.dat\n\n")
    if rerun: f.write("include orders.dat\n\n")

    ### stage-par
    f.write("stage-par:\n")

    if not rerun:
        f.write("\tCosmospar.py -c --cosmos%s\n" % (cosmos))
        ##else: f.write("\tCosmospar.py -c\n")
    #for k in range(2): f.write("\tCosmospar.py -nb -e %slambda ${%slambda} --cosmos%s\n" % (["min","max"][k],["min","max"][k],cosmos))
    #for k in range(2): f.write("\tCosmospar.py -nb -e lambda%i ${%slambda} --cosmos%s\n" % ([0,1][k],["min","max"][k],cosmos))

    if skylines: f.write("\tCosmospar.py -nb -e linelist %s --cosmos%s\n" % (skylines,cosmos))
    if comlines: f.write("\tCosmospar.py -nb -f align-mask-start.par lamfile %s --cosmos%s\n" % (comlines,cosmos))
    else:
        if instrument in ("imacs","imacs2"): inst = "imacs.dat"  
        elif instrument in ("ldss3"): inst = "ldss3.dat"
        f.write("\tCosmospar.py -nb -f align-mask-start.par lamfile %s --cosmos%s\n" % ("_".join(["afea",inst]),cosmos))
    if dp: f.write("\tCosmospar.py -nb -e deltalam %s --cosmos%s\n" % (dp,cosmos))
    if Hinstrument in ("LDSS3-Two","LDSS3-Four"):
        f.write("\tCosmospar.py -nb -f biasflat.par dewar LDSS3 --cosmos%s\n" % (cosmos))
        Hprefix = Hinstrument.split("-")[-1]
        if Hprefix == "Two": Hindex = 0
        if Hprefix == "Four": Hindex = 1
        f.write("\tCosmospar.py -nb -f stitch.par dewar %s --cosmos%s\n" % (["LDSS3-2","LDSS-4"][Hindex],cosmos))

    f.write("\tparUpdate.py -s1\n")
    f.write("\ttouch stage-par\n\n")

    nlamp = ilamps[ulamps[0]]
 
    ### stage-stitch
    if Hinstrument in ("LDSS3-Two","LDSS3-Four"):
        f.write("stage-stitch: stage-par\n# Stitch LDSS3 fits files together\n\techo Stage: Stitch LDSS3 fits files\n")
        objects = nsciences + iflats + ilamps
        for object in objects: f.write("\tstitch %s\n" % (object))
        f.write("\ttouch stage-stitch\n\n")

    ### stage-long-camera
    # do I need a direct-image alignment stage?
    if longcamera:
        f.write("stage-long-camera: stage-par\n# Run longslit.py\n\techo Stage: Run longslit.py\n")
        #f.write("\tlongslit.py %s.obsdef %s.dewoff %.3f %.3f\n" % (directory,dewfile,ddegree,increment))
        f.write("\tlongslit.py --obsdef %s.obsdef --amplitude %.3f --increment %.3f --binning %s\n" % (directory,ddegree,increment,binning))
        f.write("\ttouch stage-long-camera\n\n")

    ### stage-align-mask
    if rerun: 
        if Hinstrument in ("LDSS3-Two","LDSS3-Four"): f.write("stage-align-mask: stage-stitch\n# Skipping align-mask\n\techo Stage: Skipping align-mask\n")
        elif longcamera: f.write("stage-align-mask: stage-long-camera\n# Skipping align-mask\n\techo Stage: Skipping align-mask\n")
        else: f.write("stage-align-mask: stage-par\n# Skipping align-mask\n\techo Stage: Skipping align-mask\n")
    else:
        if Hinstrument in ("LDSS3-Two","LDSS3-Four"): f.write("stage-align-mask: stage-stitch\n# Run align-mask\n\techo Stage: Run align-mask\n")
        elif longcamera: f.write("stage-align-mask: stage-long-camera\n# Run align-mask\n\techo Stage: Run align-mask\n")
        else: f.write("stage-align-mask: stage-par\n# Run align-mask\n\techo Stage: Run align-mask\n")
        for lamp in nlamps:
            f.write("\tcp align-mask-start.par align-mask.par\n")
            f.write("\talign-mask -o %s -f %s\n" % (directory,ilamps[lamp]))
            f.write("\tsed 's/%s/%s/' %s.obsdef > %s-%s.obsdef\n" % (dewfile,ilamps[lamp],directory,directory,ilamps[lamp]))
            f.write("\talign-mask.py %s-%s %s 30\n" % (directory,ilamps[lamp],ilamps[lamp]))
            ### Temporary fix for map-spectra not creating the map in the middle of the each slit
            f.write("\tcp align-mask-start-middle.par align-mask.par\n")
            f.write("\talign-mask -o %s-%s -f %s\n" % (directory,ilamps[lamp],ilamps[lamp]))
            f.write("\talign-mask.py %s-%s %s 7\n" % (directory,ilamps[lamp],ilamps[lamp]))
    f.write("\ttouch stage-align-mask\n\n")


    #### stage-align-mask
    #if rerun: 
    #    if Hinstrument in ("LDSS3-Two","LDSS3-Four"): f.write("stage-align-mask: stage-stitch\n# Skipping align-mask\n\techo Stage: Skipping align-mask\n")
    #    elif longcamera: f.write("stage-align-mask: stage-long-camera\n# Skipping align-mask\n\techo Stage: Skipping align-mask\n")
    #    else: f.write("stage-align-mask: stage-par\n# Skipping align-mask\n\techo Stage: Skipping align-mask\n")
    #else:
    #    if Hinstrument in ("LDSS3-Two","LDSS3-Four"): f.write("stage-align-mask: stage-stitch\n# Run align-mask\n\techo Stage: Run align-mask\n")
    #    elif longcamera: f.write("stage-align-mask: stage-long-camera\n# Run align-mask\n\techo Stage: Run align-mask\n")
    #    else: f.write("stage-align-mask: stage-par\n# Run align-mask\n\techo Stage: Run align-mask\n")
    #    for z,lamp in enumerate(nlamps):
    #        print z,lamp,ilamps[lamp],nlamps
    #        f.write("\tcp align-mask-start.par align-mask.par\n")
    #        if z:
    #          #f.write("\tcp %s-%s.obsdef %s-%s.obsdef\n" % (directory,lamp0,directory,ilamps[lamp]))
    #          f.write("\talign-mask -o %s-%s -f %s\n" % (directory,lamp0,ilamps[lamp]))
    #        else:
    #          lamp0 = ilamps[lamp]
    #          f.write("\talign-mask -o %s -f %s\n" % (directory,ilamps[lamp]))
    #          #f.write("\tsed 's/%s/%s/' %s.obsdef > %s-%s.obsdef\n" % (dewfile,lamp0,directory,directory,lamp0))
    #        f.write("\tsed 's/%s/%s/' %s.obsdef > %s-%s.obsdef\n" % (dewfile,ilamps[lamp],directory,directory,ilamps[lamp]))
    #        f.write("\talign-mask.py %s-%s %s 30\n" % (directory,ilamps[lamp],ilamps[lamp]))
    #        ### Temporary fix for map-spectra not creating the map in the middle of the each slit
    #        f.write("\tcp align-mask-start-middle.par align-mask.par\n")
    #        f.write("\talign-mask -o %s-%s -f %s\n" % (directory,ilamps[lamp],ilamps[lamp]))
    #        f.write("\talign-mask.py %s-%s %s 7\n" % (directory,ilamps[lamp],ilamps[lamp]))
    #f.write("\ttouch stage-align-mask\n\n")



    ### stage-map-spectra
    f.write("stage-map-spectra: stage-align-mask\n# Run map-spectra\n\techo Stage: Run map-spectra\n")
    for lamp in nlamps:
        f.write("\tmap-spectra %s-%s\n" % (directory,ilamps[lamp]))
    f.write("\ttouch stage-map-spectra\n\n")

    ### stage-adjust-map
    f.write("stage-adjust-map: stage-map-spectra\n# Run adjust-map\n\techo Stage: Run adjust-map\n")
    for w,lamp in enumerate(nlamps):
        f.write("\tadjust-map -m %s-%s -f %s\n" % (directory,ilamps[lamp],ilamps[lamp]))
        f.write("\tcp %s.map %s_1p.map\n" % (ilamps[lamp],ilamps[lamp]))
        if testbuild:
            f.write("\tspectral-map -m %s_1p -l %s -b 1\n" % (ilamps[lamp],["HeNeAr.dat",skylines][bool(skylines)]))
        f.write("\tCosmosMarkLines.py -m %s_1p.map -id %s --%s -b %s\n" % (ilamps[lamp],["HeNeAr.dat",skylines][bool(skylines)],instrument,binning))  ### NEED TO ADD BINNING OPTION, ADD BINNING TO DATABASE
        mvline = "mv chip%i.lin chip%i-" + str(w) + ".lin"
        pyline = "".join(["python -c 'import os;[os.system(\"",mvline,"\" % (i,i)) for i in range(1,","%i" % (nfits),")]'"])
        f.write("\t%s\n" % (pyline))
    f.write("\ttouch stage-adjust-map\n\n")

    ### stage-display-lamp
    for w,lamp in enumerate(nlamps):
        f.write("stage-display-lamp-%i:\n# Display wavelength calibration %i\n\techo Stage: Display wavelength calibration %i\n" % (w,w,w))
        if instrument in ("imacs","imacs2"): 
            f.write("\tvfits %sc?.fits -c chip\?-%i.lin\n\n" % (ilamps[lamp],w))
        elif instrument in ("ldss3"): 
            f.write("\tvfits %s.fits -c chip\?-%i.lin\n\n" % (ilamps[lamp],w))

    ### stage-sflats
    f.write("stage-sflats: stage-adjust-map\n# Run Sflats\n\techo Stage: Run Sflats\n")
    if mflats == 1:
        for kflat in nflats:
            flat = iflats[kflat]
            lamp = "ccd%04d" % (take(jlamps,ulamps)[argmin(abs(jflats[kflat]-take(jlamps,ulamps)))])
            f.write("\tSflats -m %s -f %s\n" % (lamp,flat))
    elif mflats == 2:
        for gflat in gflats:
            flat0 = iflats[gflat[0]]
            flat1 = iflats[gflat[1]]
            lamp = "ccd%04d" % (take(jlamps,ulamps)[argmin(abs(jflats[gflat[0]]-take(jlamps,ulamps)))])
            f.write(" ".join(["\tSflats -m",lamp,"-f",flat0,flat1,"\n"]))
    f.write("\ttouch stage-sflats\n\n")

    ### stage-biasflat
    f.write("stage-biasflat: stage-sflats\n# Run biasflat\n\techo Stage: Run biasflat\n")
    for j,science in enumerate(nsciences):
        if mflats == 1: flat_f = iflats[uflats[j]] + "_flat"
        if mflats == 2: flat_f = iflats[cflats[j][0]]+"-"+iflats[cflats[j][1]][-1]+"_flat"
        f.write("\tbiasflat -f %s %s\n" % (flat_f,science)) # Flatten science
    f.write("\ttouch stage-biasflat\n\n")

    ### stage-subsky
    f.write("stage-subsky: stage-biasflat\n# Run subsky\n\techo Stage: Run subsky\n")
    if optimal and not testbuild:
        for k,science in enumerate(nsciences):
            f.write("\timacsSlitHole.py %s.SMF %s.map\n" % (smf,ilamps[ulamps[k]]))
            science1 = science + "_f"
            science2 = science + "_s"
            f.write("\tcp subsky-star.par subsky.par\n")
            f.write("\tsubsky -m %s_hole -f %s\n" % (ilamps[ulamps[k]],science1))
            f.write("\trm subsky.par\n")
            if Hinstrument in ("LDSS3-Two","LDSS3-Four"):
                pyline = "mv " + science2 + ".fits " + science2 + "_hole.fits"
            else: 
                mvline = "mv " + science2 + "_c%i.fits " + science2 + "_hole_c%i.fits"
                pyline = "".join(["python -c 'import os;[os.system(\"",mvline,"\" % (i,i)) for i in range(1,","%i" % (nfits),")]'"])
            f.write("\t%s\n" % (pyline))
            f.write("\tcp subsky-galaxy.par subsky.par\n")
            f.write("\tsubsky -m %s_slit -f %s\n" % (ilamps[ulamps[k]],science1))
            f.write("\trm subsky.par\n")
            if Hinstrument in ("LDSS3-Two","LDSS3-Four"):
                pyline = "mv " + science2 + ".fits " + science2 + "_slit.fits"
            else: 
                mvline = "mv " + science2 + "_c%i.fits " + science2 + "_slit_c%i.fits"
                pyline = "".join(["python -c 'import os;[os.system(\"",mvline,"\" % (i,i)) for i in range(1,","%i" % (nfits),")]'"])
            f.write("\t%s\n" % (pyline))
            #f.write("\timacsSlitHoleCombine.py %s_hole %s_slit %i\n" % (science2,science2,nfits))
            holeline = " ".join(["\timacsSlitHoleCombine.py","%s_hole" % (science2),"%s_slit" % (science2),"-%s\n" % (instrument)])
            f.write(holeline)
        f.write("\ttouch stage-subsky\n\n")
    else:
        f.write("\tcp subsky-star.par subsky.par\n")
        for k,science in enumerate(nsciences):
            science1 = science + "_f"
            f.write("\tsubsky -m %s -f %s\n" % (ilamps[ulamps[k]],science1))
        f.write("\ttouch stage-subsky\n\n")

    ### stage-extract-2dspec
    f.write("stage-extract-2dspec: stage-subsky\n# Run extract-2dspec\n\techo Stage: extract-2dspec\n")
    for k,science in enumerate(nsciences):
        science2 = science + "_s"
        f.write("\textract-2dspec -m %s -f %s\n" % (ilamps[ulamps[k]],science2))
    f.write("\ttouch stage-extract-2dspec\n\n")

    if testbuild:
        if len(nsciences) > 1:
            ### stage-sumspec
            f.write("stage-sumspec: stage-extract-2dspec\n")
            f.write("# Run sumspec\n\techo Stage: Run sumspec\n")
            sciencestring = ""
            for science in nsciences:
                sciencestring = sciencestring + science + " "
            f.write("\tsumspec -o %s_2spec %s\n" % (directory,sciencestring))
            f.write("\ttouch stage-sumspec\n\n")

        ### stage-mosaic
        f.write("stage-mosaic:\n")
        objects = nsciences + iflats + ilamps
        for obj in objects: f.write("\tmosaic %s\n" % (obj))
        f.write("\ttouch stage-mosaic\n\n")
        
        ### stage-perl
        f.write("stage-perl:\n")
        f.write("\tdefineobs\n")
        f.write("\tscd\n")
        f.write("\tscodir\n")
        f.write("\tviewspectra\n")
        f.write("\ttouch stage-perl\n\n")

        ### stage-iraf
        f.write("stage-iraf:\n")
        f.write("\ttouch stage-iraf\n\n")
        
        #f.close()
        #os.chmod(makefile,493)
        #return
    
    else:
        
        ### stage-par-2
        f.write("stage-par-2:\n")
        #f.write("\tCosmospar.py -nb -f subsky-star.par exclude ${exclude1}\n")
        #f.write("\tCosmospar.py -nb -f subsky-star.par medbox ${medbox1}\n")
        #f.write("\tCosmospar.py -nb -f subsky-galaxy.par exclude ${exclude2}\n")
        #f.write("\tCosmospar.py -nb -f subsky-galaxy.par medbox ${medbox2}\n")
        f.write("\tparUpdate.py -s3\n")
        f.write("\ttouch stage-par-2\n\n")
 
        ### stage-2pass
        f.write("stage-2pass: stage-par-2\n")
        f.write("# Run 2pass\n\techo Stage: Run 2pass\n")
        for k,science in enumerate(nsciences):
            if rerun: f.write("\timacsSpec1d.py -a ${aperature} -s %s_2spec.fits -m %s.SMF --smf -o ${ox%i},${oy%i} -k %i --edge ${edge}\n" % (science,smf,k,k,k))
            else: f.write("\timacsSpec1d.py -a ${aperature} -s %s_2spec.fits -m %s.SMF --smf -k %i --edge ${edge}\n" % (science,smf,k))
            lamp_f = ilamps[ulamps[k]]
          
            f.write("\tsed 's/%s/%s/' %s-%s.obsdef > %s.obsdef\n" % (smf,science,directory,lamp_f,science))
        f.write("\ttouch stage-2pass\n\n")
 
        ### stage-map-spectra-2
        f.write("stage-map-spectra-2: stage-2pass\n")
        f.write("# Run map-spectra-2\n\techo Stage: Run map-spectra-2\n")
        for k,science in enumerate(nsciences):
            f.write("\tmap-spectra %s\n" % (science))
        f.write("\ttouch stage-map-spectra-2\n\n")
 
        ### stage-adjust-map-2
        f.write("stage-adjust-map-2: stage-map-spectra-2\n")
        f.write("# Run adjust-map-2\n\techo Stage: Run adjust-map-2\n")
        for k,science in enumerate(nsciences):
            f.write("\tadjust-map -m %s -f %s\n" % (science,ilamps[ulamps[k]]))
            adjmap = "%s_2p" % (science)
            f.write("\tmv %s.map %s.map\n" % (ilamps[ulamps[k]],adjmap))
        f.write("\ttouch stage-adjust-map-2\n\n")
 
        ### stage-subsky-2
        f.write("stage-subsky-2: stage-adjust-map-2\n")

        if split:
            for k,science in enumerate(nsciences):
                adjmap = "%s_2p" % (science)
                science1 = science + "_f"
                science3 = science + "_2p_f"
                science4 = science + "_2p_s"
                f.write("# Run subsky on  holes and slits for %s\n\techo Stage: Run subsky on holes and slits for %s\n" % (science,science))
                f.write("\timacsSlitHole.py %s.SMF %s.map\n" % (science,adjmap))
                if Hinstrument in ("LDSS3-Two","LDSS3-Four"):
                    pyline = "cp " + science1 + ".fits " + science3 + ".fits"
                else: 
                    mvline = "cp " + science1 + "_c%i.fits " + science3 + "_c%i.fits"
                    pyline = "".join(["python -c 'import os;[os.system(\"",mvline,"\" % (i,i)) for i in range(1,","%i" % (nfits),")]'"])
                f.write("\t%s\n" % (pyline))
                f.write("\tcp subsky-star.par subsky.par\n")
                f.write("\tsubsky -m %s_hole -f %s\n" % (adjmap,science3))
                f.write("\trm subsky.par\n")
                if Hinstrument in ("LDSS3-Two","LDSS3-Four"):
                    pyline = "mv " + science4 + ".fits " + science4 + "_hole.fits"
                else: 
                    mvline = "mv " + science4 + "_c%i.fits " + science4 + "_hole_c%i.fits"
                    pyline = "".join(["python -c 'import os;[os.system(\"",mvline,"\" % (i,i)) for i in range(1,","%i" % (nfits),")]'"])
                f.write("\t%s\n" % (pyline))
                f.write("\tcp subsky-galaxy.par subsky.par\n")
                f.write("\tsubsky -m %s_slit -f %s\n" % (adjmap,science3))
                f.write("\trm subsky.par\n")
                if Hinstrument in ("LDSS3-Two","LDSS3-Four"):
                    pyline = "mv " + science4 + ".fits " + science4 + "_slit.fits"
                else: 
                    mvline = "mv " + science4 + "_c%i.fits " + science4 + "_slit_c%i.fits"
                    pyline = "".join(["python -c 'import os;[os.system(\"",mvline,"\" % (i,i)) for i in range(1,","%i" % (nfits),")]'"])
                f.write("\t%s\n" % (pyline))
                #f.write("\timacsSlitHoleCombine.py %s_hole %s_slit\n" % (science4,science4))
                holeline = " ".join(["\timacsSlitHoleCombine.py","%s_hole" % (science4),"%s_slit" % (science4),"-%s\n" % (instrument)])
                f.write(holeline)
        else:
            f.write("\tcp subsky-star.par subsky.par\n")
            for k,science in enumerate(nsciences):
                adjmap = "%s_2p" % (science)
                science1 = science + "_f"
                science3 = science + "_2p_f"
                if Hinstrument in ("LDSS3-Two","LDSS3-Four"):
                    pyline = "cp " + science1 + ".fits " + science3 + ".fits"
                else: 
                    mvline = "cp " + science1 + "_c%i.fits " + science3 + "_c%i.fits"
                    pyline = "".join(["python -c 'import os;[os.system(\"",mvline,"\" % (i,i)) for i in range(1,","%i" % (nfits),")]'"])
                f.write("\t%s\n" % (pyline))
                f.write("\tsubsky -m %s -f %s\n" % (adjmap,science3))
        f.write("\ttouch stage-subsky-2\n\n")
 
        ### stage-extract-2dspec-2
        f.write("stage-extract-2dspec-2: stage-subsky-2\n")
        f.write("# Run extract-2dspec-2\n\techo Stage: Run extract-2dspec-2\n")
        for k,science in enumerate(nsciences):
            adjmap = "%s_2p" % (science)
            science4 = science + "_2p_s"
            f.write("\textract-2dspec -m %s -f %s\n" % (adjmap,science4))
        f.write("\ttouch stage-extract-2dspec-2\n\n")
 
        ### stage-sumspec
        f.write("stage-sumspec: stage-extract-2dspec-2\n")
        f.write("# Run sumspec\n\techo Stage: Run sumspec\n")
        sciencestring = ""
        for science in nsciences:
            science5 = science + "_2p"
            sciencestring = sciencestring + science5 + " "
        #mask5 = mask + "_2p_2spec"
        f.write("\timacsSumSpec.py -o %s -f %s -c ${clipping}\n" % (directory,sciencestring))
        f.write("\ttouch stage-sumspec\n\n")
 
        ### stage-extract-flat
        f.write("stage-extract-flat: stage-sumspec\n# Run extract-flat\n\techo Stage: Run extract-flat\n")
        if mflats == 1: fflat = iflats[uflats[0]]
        if mflats == 2: fflat = iflats[gflats[0][0]]+"-"+iflats[gflats[0][1]][-1]
        f.write("\tbiasflat -f %s_flat %s\n" % (fflat,fflat))
        f.write("\textract-2dspec -m %s_2p -f %s_f\n" % (nsciences[0],fflat))
        f.write("\ttouch stage-extract-flat\n\n")

        ### stage-extract-lamp
        if extractlamp:
            f.write("stage-extract-lamp: stage-extract-flat\n# Run extract-lamp\n\techo Stage: Run extract-lamp\n")
            elamp = ilamps[0]
            f.write("\tbiasflat -f %s_flat %s\n" % (fflat,elamp))
            f.write("\textract-2dspec -m %s_2p -f %s_f\n" % (nsciences[0],elamp))
            f.write("\timacsSpec2d.py -l %s_2spec.fits\n" % (elamp))
            f.write("\tmv %s_lig.fits %s_lig.fits\n" % (elamp,directory))
            f.write("\ttouch stage-extract-lamp\n\n")

        ### stage-extract-1dspec
        if extractlamp: f.write("stage-extract-1dspec: stage-extract-lamp\n# Run extract-1dspec\n\techo Stage: Run extract-1dspec\n")
        else: f.write("stage-extract-1dspec: stage-extract-flat\n# Run extract-1dspec\n\techo Stage: Run extract-1dspec\n")

        if mflats == 1: fflat = iflats[uflats[0]]
        if mflats == 2: fflat = iflats[gflats[0][0]]+"-"+iflats[gflats[0][1]][-1]
        elamp = ilamps[0]

        if extractlamp: f.write("\timacsSpec1d.py -a ${aperature} -s %s_big.fits -m %s.SMF -f %s_2spec.fits --big --edge ${edge} -l %s_2spec.fits\n" % (directory,nsciences[0],fflat,elamp))
        else: f.write("\timacsSpec1d.py -a ${aperature} -s %s_big.fits -m %s.SMF -f %s_2spec.fits --big --edge ${edge}\n" % (directory,nsciences[0],fflat))
        f.write("\ttouch stage-extract-1dspec\n\n")
 
        ### stage-display-2p
        adjmap0 = "%s_2p.map" % (nsciences[0])
        f.write("stage-display-2p:\n# Run display\n\techo Stage: Run display\n")
        f.write("\timacsDisplay.py -1d %s_2p_1spec.fits -big %s_big.fits -dict %s_redshift.dict -t %s.trace -map %s\n\n" % (directory,directory,directory,directory,adjmap0))
 
        ### stage-flux
        dir2 = directory + "_2p"
        f.write("stage-flux: stage-extract-1dspec\n# Run flux\n\techo Stage: Run flux\n")
        #f.write("\timacsSpecFlux.py -f %s_1spec.fits -m %s.SMF -o ${ordernum} -s1 ${slitnum1} -s2 ${slitnum2}\n" % (dir2,nsciences[0]))
        ### Possible fix for the new stage-flux 
        f.write("\timacsSpecFlux.py -f %s_1spec.fits -m %s.SMF\n" % (dir2,nsciences[0]))
        f.write("\ttouch stage-flux\n\n")
 
        ### stage-display
        adjmap0 = "%s_2p.map" % (nsciences[0])
        f.write("stage-display:\n# Run display\n\techo Stage: Run display\n")
        f.write("\timacsDisplay.py -1d %s_1spec.fits -big %s_big.fits -dict %s_redshift.dict -t %s.trace -map %s\n\n" % (directory,directory,directory,directory,adjmap0))
 
        ### stage-redshift
        f.write("stage-redshift:\n# Run redshift\n\techo Stage: Run redshift\n")
        f.write("\taRedshift.py %s_1spec.fits -nz 100 -imacs -input %s_redshift.dict\n" % (directory,directory))
        f.write("\ttouch stage-redshift\n\n")
 
        ### stage-redshift-display
        f.write("stage-redshift-display:\n# Run redshift display\n\techo Stage: Run redshift display\n")
        f.write("\taRedshiftDisplay.py -bin %s_redshiftCC.bin -1d %s_1spec.fits -big %s_big.fits -dict %s_redshift.dict -t %s.trace\n\n" % (directory,directory,directory,directory,directory))
 
        ### stage-redshift-output
        f.write("stage-redshift-output:\n# Run redshift output\n\techo Stage: Run redshift output\n")
        f.write("\taRedshiftOutput.py -bin %s_redshiftCC.bin -dict %s_redshift.dict -smf %s.SMF -fits %s_1spec.fits\n" % (directory,directory,smf,directory))
        f.write("\ttouch stage-redshift-output\n\n")

    ### clean-stage1
    f.write("clean-stage1:\n")
    f.write("\trm -f %s\n\n" % (stage1))

    ### clean-stage2
    f.write("clean-stage2:\n")
    f.write("\trm -f %s\n\n" % (stage2))

    if not testbuild:
        ### clean-stage3
        f.write("clean-stage3:\n")
        f.write("\trm -f %s\n\n" % (stage3))
 
        ### clean-stage4
        f.write("clean-stage4:\n")
        f.write("\trm -f %s\n\n" % (stage4))
 
        ### clean-stage5
        f.write("clean-stage5:\n")
        f.write("\trm -f %s\n\n" % (stage5))
 
        ### clean-stage6
        f.write("clean-stage6:\n")
        f.write("\trm -f %s\n\n" % (stage6))

    ### clean
    f.write("clean:\n")
    f.write("\trm ccd*_flat*.fits\n")
    f.write("\trm ccd*_f*.fits\n")
    f.write("\trm ccd*_s*.fits\n\n")

    ### very-clean
    f.write("very-clean:\n")
    f.write("\trm ccd*_2spec.fits\n")
    f.write("\trm ccd*.dewoff*\n")
    f.write("\trm *.map\n")
    f.write("\trm *.ps\n")
    f.write("\trm *.par*\n")
    f.write("\trm *.xy\n")
    f.write("\trm ccd*.obsdef\n")
    f.write("\trm ccd*.SMF\n")
    f.write("\trm -f %s\n\n" % (all))

    ### par-update
    f.write("par-update:\n")
    f.write("\tparUpdate.py -s1 -s2 -s3\n")
    f.close()
    os.chmod(makefile,493)

def database(db,pdict,instrument):
    datalist = [dbline for dbline in open(db).readlines() if "#" not in dbline[0]]

    if instrument in ("imacs","imacs2"):
        FILENAME_POS = 0
        DATE_POS = 1
        INSTRUMENT_POS = 3
        DISPERSR_POS = 5
        G_ANGLE_POS = 6
        MASK_POS = 7
        DEWARORI_POS = 9
        OBJECT_POS = 10
    elif instrument in ("ldss3"):
        FILENAME_POS = 0
        DATE_POS = 1
        INSTRUMENT_POS = 3
        # GRISM
        DISPERSR_POS = 5
        # None
        G_ANGLE_POS = None
        # None
        DEWARORI_POS = None 
        MASK_POS = 6 
        OBJECT_POS = 8

    L = []
    objs = []
    for dataline in datalist:
        datall = dataline.split("\t")
        if "f/2-imaging" in datall[DISPERSR_POS].lower(): continue
        if "open" in datall[DISPERSR_POS].lower(): continue

        #nobj = datall[OBJECT_POS].split()[0]+"_"+datall[DISPERSR_POS]
        nobj = datall[MASK_POS]
        test = 0
        for obj in objs:
            if nobj in obj:
               test = 1
               break
        if not test and [word for word in BLACKLIST]:
            objs.append(nobj)

    #print objs

    D = {}
    for obj in objs:
        flats = []
        lamps = []
        sciences = []
        for dataline in datalist:
            datall = dataline.split("\t")
            if "f/2-imaging" in datall[DISPERSR_POS].lower(): continue
            if "open" in datall[DISPERSR_POS].lower(): continue

            #tobj = datall[OBJECT_POS].split()[0]+"_"+datall[DISPERSR_POS]
            tobj = datall[MASK_POS]
            if obj in tobj:
                #print obj
                #print datall[OBJECT_POS].lower()
                if [word for word in FLATLIST if word in datall[OBJECT_POS].lower()]:
                    test = 0
                    for flat in flats:
                        if flat in datall[FILENAME_POS][:-7]:
                           test = 1
                           break
                    if not test:
                        flats.append(datall[FILENAME_POS][:-7])
                    
                if [word for word in LAMPLIST if word in datall[OBJECT_POS].lower()]:
                    test = 0
                    for lamp in lamps:
                        if lamp in datall[FILENAME_POS][:-7]:
                           test = 1
                           break
                    if not test:
                        lamps.append(datall[FILENAME_POS][:-7])
        
                if not [word for word in BLACKLIST if word in datall[OBJECT_POS].lower()]:
                    test = 0
                    for science in sciences:
                        if science in datall[FILENAME_POS][:-7]:
                           test = 1
                           break
                    if not test: 
                        sciences.append(datall[FILENAME_POS][:-7])
                if DISPERSR_POS: disperser = datall[DISPERSR_POS]
                if G_ANGLE_POS: g_angle = datall[G_ANGLE_POS]
                if DEWARORI_POS: dewarori = datall[DEWARORI_POS] 
                if MASK_POS: mask = datall[MASK_POS]
                if DATE_POS: tdate = datall[DATE_POS]
                if INSTRUMENT_POS: Hinstrument = datall[INSTRUMENT_POS]
        if instrument in ("imacs","imacs2"):
            D[obj]={"flats":flats,"lamps":lamps,"sciences":sciences,"disperser":disperser,"dewarori":dewarori,"mask":mask,"date":tdate,"Hinstrument":Hinstrument,"g-angle":g_angle}
        elif instrument in ("ldss3"):
            D[obj]={"flats":flats,"lamps":lamps,"sciences":sciences,"disperser":disperser,"mask":mask,"date":tdate,"Hinstrument":Hinstrument}
    if pdict: print;print D;print
    return D

def includes(wrange,exclude1,exclude2,medbox1,medbox2,siglimit1,siglimit2,spline2d1,spline2d2,search_height,search_width,slit_width,ord_sag,ord_disp):
    f = open("parameters.dat","w")
    f.write("# Gus's sky-subtraction parameters\n\n")
    f.write("# minlambda - minimum wavelength\n")
    f.write("# maxlambda - maximum wavelength\n")
    f.write("# search_height - adjust-map: half height of search range\n")
    f.write("# search_width - adjust-map: half-width of search range\n")
    f.write("# slit_width - adjust-map: slit width\n")
    f.write("# ord_disp - adjust-map: order of dispersion error fit\n")
    f.write("# ord_sag - adjust-map: order of spectrum y position error fit\n")
    f.write("# exclude1 - subsky: half width of exclusion region around holes\n")
    f.write("# exclude2 - subsky: half width of exclusion region around slits\n")
    f.write("# spline2d1 - subsky: Perform 2-d spline fit to spectra for holes\n")
    f.write("# spline2d2 - subsky: Perform 2-d spline fit to spectra for slits\n")
    f.write("# medbox1 - subsky: boxsize for running median CR rejection around holes\n")
    f.write("# medbox2 - subsky: boxsize for running median CR rejection around slits\n")
    f.write("# siglimit1 - subsky: CR rejection threshold in sigmas for holes\n")
    f.write("# siglimit2 - subsky: CR rejection threshold in sigmas for slits\n\n")
    f.write("# Pipeline extraction and fluxing parameters\n\n")
    f.write("# clipping - cosmic ray clipping value used in summing spectra\n")
    f.write("# edge - edge excluded from fits\n")
    f.write("# aperature - aperature size for 1d extraction\n")
    f.write("# slitnum1 - reference object to use absorbtion features\n")
    f.write("# slitnum2 - reference object to fit for slope\n")
    f.write("# ordernum - order of legendre polynomial to fit reference objects slope\n\n")
    f.write("# For optimal fluxing, ensure that the wavelength ranges 6850-6960, 7570-7700,\n# 7140-7360 and 9270-9770 do not fall on a chip gap or empty data.\n\n")
    f.write("# stage1\n")
    if wrange: min,max = wrange
    else: min,max = "3500","10000"
    f.write("minlambda = %s\n" % (min))
    f.write("maxlambda = %s\n" % (max))
    if not search_height: search_height = "4"
    f.write("search_height = %s\n" % (search_height))
    if not search_width: search_width = "2"
    f.write("search_width = %s\n" % (search_width))
    if not slit_width: slit_width = "6"
    f.write("slit_width = %s\n" % (slit_width))
    if not ord_disp: ord_disp = "3"
    f.write("ord_disp = %s\n" % (ord_disp))
    if not ord_sag: ord_sag = "2"
    f.write("ord_sag = %s\n" % (ord_sag))
   
    f.write("# stage2\n# box size =\n# slit size =\n")
    if not exclude1: exclude1 = "2"
    f.write("exclude1 = %s\n" % (exclude1))
    if not exclude2: exclude2 = "3"
    f.write("exclude2 = %s\n" % (exclude2))
    if spline2d1: spline2d1 = "'y'"
    else: spline2d1 = "'n'"
    f.write("spline2d1 = %s\n" % (spline2d1))
    if spline2d2: spline2d2 = "'y'"
    else: spline2d2 = "'n'"
    f.write("spline2d2 = %s\n" % (spline2d2))
    if not medbox1: medbox1 = "15"
    f.write("medbox1 = %s\n" % (medbox1))
    if not medbox2: medbox2 = "15"
    f.write("medbox2 = %s\n" % (medbox2))
    if not siglimit1: siglimit1 = "8"
    f.write("siglimit1 = %s\n" % (siglimit1))
    if not siglimit2: siglimit2 = "10"
    f.write("siglimit2 = %s\n" % (siglimit2))
    f.write("edge = 3\n")
    f.write("clipping = 15\n")
    f.write("aperature = 10\n")
    f.write("# stage3\n")
    f.write("slitnum1 = 0\n")
    f.write("slitnum2 = 0\n")
    f.write("ordernum = 3\n")
    f.close()

def irafcl(mask,lamp):
    mask1 = mask + "-1"
    f = open(mask1 + ".cl","w")
    f.write("# %s IRAF script\n" % (mask))
    f.write("!ds9 &\n")
    f.write("display8 %s 1 z1=0 z2=5000\n" % (lamp))
    f.write("tvmark 1 %s.xy\n" % (mask1))
    f.close()

def obsdef(mask,dewfile,gratingv,cosmos,instrument,smf,gr_angle):
    f = open(mask + ".obsdef","w")
    f.write("%-15s %s\n" % ("GR_ANGLE",gr_angle))

    ### Grating/Grism
    # May need a better way to deal with *.obsdef?!
    if "-" in gratingv: splitv = "-" 
    elif "_" in gratingv: splitv = "_"

    if instrument in ("imacs","imacs2"):
        if len(gratingv.split(splitv)) > 1:
            gratingv = gratingv.split(splitv)
            if gratingv[1][-1] == "l": val = gratingv[1][:-1]
            else: val = gratingv[1]
            if "gri" in [k[:3].lower() for k in gratingv]: match = "_".join(["IMACS","grism",val]); cam = 0
            else: match = "_".join(["IMACS",val+"A"]); cam = 1
        else:
            ### MOE ###
            match = gratingv; cam = 1
 
    elif instrument in ("ldss3"):
        gratingv = gratingv.split(splitv)
        if gratingv[0].lower() == "old": val = "_".join([gratingv[1].lower(),gratingv[2].lower()])
        else: val = "".join([gratingv[0],gratingv[1].lower()])
        match = "_".join(["LDSS3",val])
    else:
        raise "Error with grating string, grating not in list."


    for k,grating in enumerate(instruments[instrument]["GRATING"]):
        if match == grating: 
            f.write("%-15s %s\n" % ("GRATING",instruments[instrument]["GRATING"][k])) 
            break

    if cosmos != '1-0': f.write("%-15s %s\n" % ("INSTRUMENT",instruments[instrument]["INSTRUMENT"])) 
    if cosmos == '1-0': f.write("%-15s %i\n" % ("GR_ORDER",0)) 
    else: f.write("%-15s %i\n" % ("GR_ORDER",instruments[instrument]["GR_ORDER"])) 
    f.write("%-15s %s\n" % ("DEWAR",instruments[instrument]["DEWAR"])) 
    if instruments[instrument]["CAMERA"]: f.write("%-15s %s\n" % ("CAMERA",instruments[instrument]["CAMERA"][cam])) 
    else: f.write("%-15s\n" % ("CAMERA"))
    f.write("%-15s %s\n" % ("DEWOFF",dewfile)) 
    f.write("%-15s %s\n" % ("MASK",smf)) 
    f.write("%-15s %s\n" % ("MODE",instruments[instrument]["MODE"])) 

    f.close()

def dewoff(dewfile):
    f = open(dewfile + ".dewoff","w")
    f.write(dewarobs[D[id]["disperser"]][dewfile])
    f.close()

#cosmos1 = 0
binning = '1x1'
cosmos = '2-04'
comlines = None
db = None
dblist = 0
ddegree = 5.0
empty_skipped = []
exclude1 = None
exclude2 = None
extractlamp = 1
increment = 0.5
instrument = None
longcamera = 0
makefile = None
medbox1 = None
medbox2 = None
mflats = 1
nobak = 1
split = 1
optimal = 0
ord_disp = None
ord_sag = None
pdict = 0
remake = 0
rerun = 0
search_height = None
search_width = None
setup = sys.argv
siglimit1 = None
siglimit2 = None
skipped = []
skyfit = 0
skylines = None
slit_width = None
spline2d1 = 1
spline2d2 = 1
targets = None
testbuild = 0
wrange = None

for n,arg in enumerate(sys.argv):
    #if arg in ("-1","--cosmos1"): cosmos1 = 1

    if arg in ("-1-0","--cosmos1-0"):  cosmos = '1-0'
    elif arg in ("-2-0","--cosmos2-0"):  cosmos = '2-0'
    elif arg in ("-2-02","--cosmos2-02"):  cosmos = '2-02'
    elif arg in ("-2-04","--cosmos2-04"):  cosmos = '2-04'

    elif arg in ("-b","--binning"):  binning = sys.argv[n+1]
    elif arg in ("-cl","--comlines"): comlines = sys.argv[n+1]
    elif arg in ("-db","--database"): db = sys.argv[n+1]
    elif arg in ("-dd","--ddegree"): ddegree = float(sys.argv[n+1])
    elif arg in ("-e1","--exclude1"): exclude1 = sys.argv[n+1]
    elif arg in ("-e2","--exclude2"): exclude2 = sys.argv[n+1]
    elif arg in ("-f","--flats"): mflats = int(sys.argv[n+1])
    elif arg in ("-inc","--increment"): increment = float(sys.argv[n+1])
    elif arg in ("-ins","--instrument"): instrument = sys.argv[n+1].lower()
    elif arg in ("-l","--list"): dblist = 1
    elif arg in ("-lc","--long-camera"): longcamera = 1
    elif arg in ("-mk","--make"): makefile = sys.argv[n+1]
    elif arg in ("-mb1","--medbox1"): medbox1 = sys.argv[n+1]
    elif arg in ("-mb2","--medbox2"): medbox2 = sys.argv[n+1]
    elif arg in ("-ns1","--no2dspline1"): spline2d1 = 0
    elif arg in ("-ns2","--no2dspline2"): spline2d2 = 0
    elif arg in ("-nb","--nobak"): nobak = 0
    elif arg in ("-el","--noextractlamp"): extractlamp = 0
    elif arg in ("-ns","--nosplit"): split = 0
    elif arg in ("-o","--optimal"): optimal = 1 
    elif arg in ("-od","--ord-disp"): ord_disp = sys.argv[n+1]
    elif arg in ("-os","--ord-sag"): ord_sag = sys.argv[n+1]
    elif arg in ("-pd","--pdict"): pdict = 1 
    elif arg in ("-rm","--remake"): remake = 1
    elif arg in ("-rr","--rerun"): rerun = 1
    elif arg in ("-sf","--skyfit"): skyfit = 1
    elif arg in ("-sh","--search-height"): search_height = sys.argv[n+1]
    elif arg in ("-sl","--skylines"): skylines = sys.argv[n+1]
    elif arg in ("-sl1","--siglimit1"): siglimit1 = sys.argv[n+1]
    elif arg in ("-sl2","--siglimit2"): siglimit2 = sys.argv[n+1]
    elif arg in ("-slw","--slit-width"): slit_width = sys.argv[n+1]
    elif arg in ("-sw","--search-width"): search_width = sys.argv[n+1]
    elif arg in ("-t","--targets"):          

        targets = []
        for k in range(n+1,len(sys.argv)):
            subtarglist = sys.argv[k]
            if subtarglist in argblacklist: break
            targets.append(subtarglist)
    elif arg in ("-tb","--test-build"): testbuild = 1
    elif arg in ("-wr","--wrange"): wrange = sys.argv[n+1].split("-")
    elif arg in ("-h","--help"):
        help()
        sys.exit()

if makefile is None:
    makefile = "Makefile"

if db is not None:
    D = database(db,pdict,instrument)
    if dblist:
        Dlist = [id for id in D if len(D[id]["flats"]) != 0 and len(D[id]["lamps"]) != 0 and len(D[id]["sciences"]) != 0]
        print Dlist
        sys.exit()
    else: 
        # check for cvs
        #rcvs,wcvs = popen2.popen2("which cvs")
        #cvs = rcvs.readlines()
        #if cvs:
        #    cvsenv = os.getenv("CVSROOT")
        #    if not cvsenv: os.putenv("CVSROOT","/data1/hidari/gwalth/cvsroot")
        #    rPipe,wPipe = popen2.popen2("which CosmosPipeline.py")
        #    Pipe = rPipe.readlines()[0]
        #    rstat,wstat = popen2.popen2("cvs status %s" % (Pipe))
        #    status = rstat.readlines()
        #    pstr = "Working revision:"
        #    for sv in status:
        #        if pstr in sv:
        #            version = sv.replace(pstr,"").strip()
        #            break
        #else:
        #    version = "Not available" 
        version = "Not available" 
        mf = open(makefile,"w")
        mf.write("# Cosmos Pipeline version %s\n\n" % (version))
        
        masklist = []
        targstring = ""
        cleanstring = ""
        parstring = ""
        
        for id in D:
            if not len(D[id]["flats"]) or not len(D[id]["lamps"]) or not len(D[id]["sciences"]):
                empty_skipped.append(id)
                continue
            if targets is not None and id not in targets: continue
            if id == "default": 
                fframe = [zz.split("/")[-1] for zz in D[id]["flats"]]
                lframe = [zz.split("/")[-1] for zz in D[id]["lamps"]]
                sframe = [zz.split("/")[-1] for zz in D[id]["sciences"]]
                frames = fframe + lframe + sframe
                frames.sort()
                #print "problem with header keyword(s), skipping frames %s\n" % (frames)
                skipped.append(frames)
                continue
            print id
            
            masklist.append(id)
            targstring = targstring + "targ_" + id + " "
            cleanstring = cleanstring + "clean_" + id + " "
            parstring = parstring + "par_" + id + " "

            #sys.exit()
            if not os.path.exists(id): os.mkdir(id)
            os.chdir(id)
            if instrument in ("imacs","imacs2"):
                nfits = 9
                if D[id]["dewarori"] == "Nod&Shuffle": ori = "NS"
                elif D[id]["dewarori"] == "Normal": ori = "N"
                for dewobs in dewarobs[D[id]["disperser"]].keys():
                    ### Special case for Normal and Nod&Shuffle Grisms
                    if len(dewobs.split("_")) > 1 and "Grism" in dewobs.split("_")[0] and dewobs.split("_")[1] == ori:
                        dewfile = dewobs
                    else: dewfile = dewobs
                ### Chip6 Date Check
                tdate = D[id]["date"].split("-")
                td = datetime.date(int(tdate[0]),int(tdate[1]),int(tdate[2]))
                if td < datetime.date(2005,8,1): instrument = "imacs"#; print td 
                else: instrument = "imacs2"
                if longcamera: gr_angle = D[id]["g-angle"]
                else: gr_angle = instruments[instrument]["GR_ANGLE"]
            elif instrument in ("ldss3"):
                nfits = 3
                dewfile = dewarobs[D[id]["disperser"]].keys()[0]
                gr_angle = instruments[instrument]["GR_ANGLE"]
            [os.symlink(frame+"c%i.fits" % (c),os.path.basename(frame+"c%i.fits" % (c))) for reduction in [D[id]["flats"],D[id]["sciences"],D[id]["lamps"]] for frame in reduction for c in range(1,nfits) if not os.path.exists(os.path.basename(frame+"c%i.fits" % (c)))] 
            if not rerun and not remake:
                includes(wrange,exclude1,exclude2,medbox1,medbox2,siglimit1,siglimit2,spline2d1,spline2d2,search_height,search_width,slit_width,ord_sag,ord_disp)
                dewoff(dewfile)
                obsdef(id,dewfile,D[id]["disperser"],cosmos,instrument,D[id]["mask"],gr_angle)
                if instrument in ("imacs","imacs2"):
                    shutil.copy2("/data1/hidari/gwalth/Cosmos/linelists/afea_imacs.dat",".")
                elif instrument in ("ldss3"):
                    shutil.copy2("/data1/hidari/gwalth/Cosmos/linelists/afea_ldss3.dat",".")
                shutil.copy2("/data1/hidari/gwalth/Cosmos/linelists/HeNeAr.dat",".")
                shutil.copy2("/data1/hidari/gwalth/Cosmos/linelists/middle.dat",".")
            reduce(id,D[id]["flats"],D[id]["sciences"],D[id]["lamps"],dewfile,"targ_" + id + ".pipeline",setup,optimal,wrange,skylines,comlines,dispersion[D[id]["disperser"]],nobak,skyfit,cosmos,instrument,rerun,nfits,D[id]["mask"],split,D[id]["Hinstrument"],mflats,version,testbuild,longcamera,ddegree,increment,binning,extractlamp)
            if os.path.exists("Makefile"): os.remove("Makefile")
            os.symlink("targ_" + id + ".pipeline","Makefile")
            #irafcl(id,D[id]["lamps"][0])

            os.chdir("..")
            print
        
        mf.write("# masks=%s\n" % (masklist))
        mf.write("# created %s\n\n" % (timestamp()))
        mf.write("all: %s\n\n" % (targstring))

        setupstring = " ".join([arg for arg in setup if not arg in ("-rr","--rerun","-rm","--remake")])

        mf.write("remake:\n")
        mf.write("\t%s --remake\n\n" % (setupstring))
        mf.write("rerun:\n")
        mf.write("\t%s --rerun\n\n" % (setupstring))
        mf.write("regenerate:\n")
        mf.write("\t%s\n\n" % (setupstring))

        makeline = "\tcd %s; make -f %s.pipeline %s; cd ../\n"
        displine = "\ndisplay%s_%s:\n\tcd %s; make -f %s.pipeline stage-display%s; cd ../\n"
        targetL = targstring.split()
        cleanL = cleanstring.split()
        parL = parstring.split()


        mf.write("stage1:\n")
        [mf.write(makeline % (t.replace("targ_",""),t,"stage1")) for t in targetL]

        mf.write("\nstage2:\n")
        [mf.write(makeline % (t.replace("targ_",""),t,"stage2")) for t in targetL]

        if testbuild: 
            mf.write("\nstage-mosaic:\n")
            [mf.write(makeline % (t.replace("targ_",""),t,"stage-mosaic")) for t in targetL]
            
            mf.write("\nstage-perl:\n")
            [mf.write(makeline % (t.replace("targ_",""),t,"stage-perl")) for t in targetL]
            
            mf.write("\nstage-iraf:\n")
            [mf.write(makeline % (t.replace("targ_",""),t,"stage-iraf")) for t in targetL]
        else:
            mf.write("\nstage3:\n")
            [mf.write(makeline % (t.replace("targ_",""),t,"stage-2pass")) for t in targetL]
            
            [mf.write(makeline % (t.replace("targ_",""),t,"stage3")) for t in targetL]

            mf.write("\nstage4:\n")
            [mf.write(makeline % (t.replace("targ_",""),t,"stage4")) for t in targetL]

            mf.write("\nstage5:\n")
            [mf.write(makeline % (t.replace("targ_",""),t,"stage5")) for t in targetL]

            mf.write("\nstage6:\n")
            [mf.write(makeline % (t.replace("targ_",""),t,"stage6")) for t in targetL]

            [mf.write(displine % ("-2p",t.replace("targ_",""),t.replace("targ_",""),t,"-2p")) for t in targetL]

            [mf.write(displine % ("",t.replace("targ_",""),t.replace("targ_",""),t,"")) for t in targetL]
            
        [mf.write("\n%s:\n\tcd %s; time make -f %s.pipeline; cd ../\n" % (t,t.replace("targ_",""),t)) for t in targetL]

        mf.write("\nclean: %s\n\n" % (cleanstring))
        [mf.write("\n%s:\n\tcd %s; make clean; cd ../\n" % (c,c.replace("clean_",""))) for c in cleanL]

        mf.write("\npar-update: %s\n\n" % (parstring))
        [mf.write("\n%s:\n\tcd %s; make par-update; cd ../\n" % (p,p.replace("par_",""))) for p in parL]

        mf.close()
        os.chmod(makefile,493)
        print "mask(s) skipped because of missing reduction frame(s):\n",empty_skipped
        print "frames(s) skipped because of bad header keyword(s):\n",skipped
else:
    help()
    sys.exit()
