#!/usr/bin/env python

from basics import *
from FITS import *
from pgplot import *
from VTKHelperFunctions import *

def khelp():
    print "\nBasic"
    print "  e\t\temission binning toggle\n  q\t\tquit\n  r\t\treset zoom and stretch\n  s\t\tstatus toggle\n  Space\t\tdisplay cursor position\n  Shift+/\thelp\n  Shift+t\ttrace display toggle\n  c\t\toverlay chip regions\n  Shift+c\tadd comments\n  Shift+`\tdisplay spectra status\n  Shift+s\tsave to file\n  Shift+l\treload fits files\n  Shift+,\tsmaller pgplot window\n  Shift+.\tlarger pgplot window\n  Shift+;\tpython command" 

    print "\nFrame"
    print "  n\t\tnext object\n  p\t\tprevious object\n  b\t\tbeginning frame\n  f\t\tending frame\n"

    print "\nZoom"
    print "  x\t\tzoom out in x\n  y\t\tzoom out in y\n  z\t\tzoom out in x and y\n  Shift+x\tzoom in x\n  Shift+y\tzoom in y\n  Shift+z\tzoom in x and y\n  h\t\tpan to -x\n  j\t\tpan to -y\n  k\t\tpan to +y\n  l\t\tpan to +x"

    print "\nStretch"
    print "  ;\t\tsharper\n  '\t\tduller\n  ,\t\tbrighter\n  .\t\tfainter\n  /\t\tinvert"

    print "\nSmoothing"
    print "  Shift+g\tGaussian smoothing"

    print "\nMask"
    print "  Shift+m\tglobal mask\n  m\t\tmask\n  d\t\tdelete masks\n  Shift+n\tmask noise\n  Shift+w\tmask negative spectra\n  o\t\tmask spectra = 0.0\n  Shift+o\tglobal mask spectra = 0.0"

    print "\nRedshift"
    print "  Shift+0\tredshift display toggle\n  [\t\tincrement in z -0.01 \n  ]\t\tincrement in z +0.01\n  Shift+[\tfine increment in z -0.0001\n  Shift+]\tfine increment in z +0.0001\n  Shift+1\tz set to 0.00\n  Shift+2\tz set to 0.10\n  Shift+3\tz set to 0.35\n  Shift+4\tz set to 0.75\n  Shift+5\tz set to 1.50\n  Shift+6\tz set to 3.00\n  Shift+7\tz set to 5.00\n  Shift+r\tmanual redshift, value set by user\n  Shift+v\trun aRedshift.py on spectra"

    print "\nTemplate"
    print "  t\t\tcreate template set\n  1\t\tgalaxy1 template set [23,27]\n  2\t\tgalaxy2 template set [23]\n  3\t\tgalaxy3 template set [27]\n  4\t\tgalaxy4 template set [24,25,26,27]\n  5\t\tgalaxy5 template set [23,24,25,26,27,28]\n  6\t\tagn template set [29,30,31,32]\n  7\t\tstar template set [4,10,12]"

def cmd_help():
    print "-1d    FITSFILE  1d spectra"
    print "-big   FITSFILE  2d spectra"
    print "-dict  DICT      python dictionary, default is redshift.dict"
    print "-d     DIRECT    directory, default is redshiftCC"
    print "-fr    FRAMENUM  frame number to start from, default is 0"
    print "-h               print program options"
    print "-i1    LOWPIX    low pixel value for 1d spectra, default is minimum extrema"
    print "-i2    HIGHPIX   high pixel value for 1d spectra, default is maximum extrema"
    print "-l     LINELIST  galaxy line list, default /data1/mifune/kelson/Python.Linux/Basics/galaxylines.dat"
    print "-map   MAPFILE   map created from Cosmos"
    print "-p     PGMODE    pgplot mode, default /xs"
    print "-t     BINFILE   trace"
    print "-w1    MINWAVE   wavelegnth value, default is minimum extrema"
    print "-w2    MAXWAVE   wavelength value, default is maximum extrema"
    print "-z1    LOWPIX    low pixel value for 2d spectra, default is -100"
    print "-z2    HIGHPIX   high pixel value for 2d spectra, default is 100"

def GetSpectra1D(row,FITS1):
    if FITS1["naxis3"] > 3:
       ll = [0,row,3]
       ur = [FITS1["naxis1"]-1,row,4]
    else:
       ll = [0,row,0]
       ur = [FITS1["naxis1"]-1,row,1]
    return MLab.squeeze(FITS1.readsec(ll,ur))

class SingleObject:
    def __init__(self,row,N,FITS1,FITS2):
        self.FITS1 = FITS1
        self.FITS2 = FITS2
        self.naxis1 = FITS1["naxis1"]
        self.naxis2 = FITS1["naxis2"]
        self.crval1 = FITS1["crval1"]
        self.cdelt1 = FITS1["cdelt1"]
        self.crpix1 = FITS1["crpix1"]
        self.dcflag = FITS1["dc-flag"]
        self.ap = FITS1["dapert"]
        self.row = row
        self.N = N
        self.waves = (arange(self.naxis1)+1-self.crpix1)*self.cdelt1+self.crval1
        if self.dcflag: self.waves = power(10,self.waves)
        self.name = self.FITS1["obj%03d" % (1+self.row)].strip()

        self.center0 = self.FITS1["cntrl%03d" % (1+self.row)]
        self.center1 = self.FITS1["excen%03d" % (1+self.row)]
        if self.center1 == "N/A": self.center1 = self.center0

        #self.center = self.FITS1["excen%03d" % (1+self.row)]
        #if self.center == "N/A": self.center = self.FITS1["cntrl%03d" % (1+self.row)]
        #print self.center
        self.csecta = self.FITS1["csect%da" % (1+self.row)]
        self.csectb = self.FITS1["csect%db" % (1+self.row)]
        self.Get1D()
        self.Get2D()
    def GetInfo(self):
        return self.w1,self.w2,self.imin,self.imax,self.waves,self.name
    def Display(self,a1,a2,w1,w2,imin,imax,obj,edata,key,smo):
        self.key = key
        if w1: self.w1 = w1
        else: self.w1 = self.waves[0]
        if w2: self.w2 = w2
        else: self.w2 = self.waves[-1]
        self.a1 = a1
        self.a2 = a2
        self.S = obj["status"]
        self.T = obj["tplate"]
        self.Ts = obj["ttype"]
        self.m = obj["mask"]
        self.bin = obj["bin"]
        self.z = obj["z"]
        self.tfun = edata["trace"]["data"],edata["trace"]["status"] 
        self.redshift = [edata["redshift"]["data"],edata["redshift"]["status"]]
        self.mapl = [edata["map"]["data"],edata["map"]["status"]]
        if imin: self.imin = imin
        else: self.imin = -3*getpct(self.noise1d,98)
        if imax: self.imax = imax
        else: self.imax = 2*getpct(self.spec1d,98)
        self.smo = smo
        self.wra = array([self.w1,self.w2])
        self.ira = array([self.imin,self.imax])
        self.Display2D()
        self.DisplayStretch()
        self.DisplayInfo()
        self.Display1D()
    def Get1D(self):
        # Spectra, Noise, Flat, Lamp, Flattened-flat, Fluxed Spectra, Fluxed Noise
        if self.FITS1["naxis3"] == 6:
           ll = [0,self.row,4]
           ur = [self.naxis1-1,self.row,5]
        # Spectra, Noise, Flat, Flattened-Flat, Fluxed Spectra, Fluxed Noise
        elif self.FITS1["naxis3"] == 5:
           ll = [0,self.row,3]
           ur = [self.naxis1-1,self.row,4]
        # Spectra, Noise, Flat, (Lamp)
        else:
           ll = [0,self.row,0]
           ur = [self.naxis1-1,self.row,1]
        self.spec1d,self.noise1d = MLab.squeeze(self.FITS1.readsec(ll,ur))
    def Get2D(self):
        ll = [0,self.csecta-1]
        ur = [self.naxis1-1,self.csectb-1]
        self.spec2d = self.FITS2.readsec(ll,ur)
    def Display1D(self):
        cpgsvp(0.10,0.90,0.15,0.65)
        if self.imin == self.imax: self.imin,self.imax = self.a1,self.a2
        newbox([self.w1,self.w2],[self.imin,self.imax],manvp=1,ch=0.7,era=0)
        xlabel("Wavelength (\A)",ch=1.0)
        ylabel("Counts",ch=1.0)
        g = self.spec1d
        if self.smo:
            if self.smo.values()[0] > 0: 
               if self.smo.keys()[0] == "B":
                   print "boxcar feature still under construction..."
                   wrttext("","(Boxcar fw="+str(self.smo.values()[0])+")",-40.0,0.6,0.8,"T",1,0,1)
               if self.smo.keys()[0] == "G":
                   g = VTKGauss(g,smo.values()[0]/2.35,numret=1)
                   wrttext("","(Gauss fw="+str(self.smo.values()[0])+")",-40.0,0.6,0.8,"T",1,0,1)
        drawln(self.waves,g,ls=1,ci=1,hist=1)
        drawln(self.waves,self.noise1d,ls=1,ci=2,hist=1)
        if self.key: self.DisplayKey()
        if self.m: self.DisplayM()
        if self.redshift[1]: self.DisplayRedshift()
    def Display2D(self):
        cpgsvp(0.10,0.90,0.65,0.85)
        newbox([self.w1,self.w2],[self.csecta-1,self.csectb-1],manvp=1,ch=0.7,xlab=0)
        ylabel("Y (pixels)",ch=1.0)
        trf = array([self.waves[0]-self.cdelt1/2.0,self.cdelt1,0.0,self.csecta-1.5,0.0,1.0])
        cpggray(self.spec2d,self.spec2d.shape[1],self.spec2d.shape[0],\
            1,self.naxis1,1,self.spec2d.shape[0],self.a1,self.a2,trf)
        if self.mapl[1]: self.DisplayChipRegions()
        if self.tfun[0] != None and self.tfun[1]:
            drawln(self.waves,self.center1+self.tfun[0],ls=2,ci=1)
            drawln(self.waves,self.center1+self.tfun[0]+self.ap/2.0,ls=2,ci=3)
            drawln(self.waves,self.center1+self.tfun[0]-self.ap/2.0,ls=2,ci=3)
            drawln(self.waves,self.center0+self.tfun[0],ls=2,ci=7)
            wrttext("","%.2f" % self.center1,-1,0.7,1.005,"T",1,0,1)
            wrttext("","%.2f" % self.center0,-2,0.7,1.005,"T",1,0,7)
    def DisplayInfo(self):
        cpgsvp(0.10,0.90,0.90,0.95)
        wrttext("","Object = "+self.name,1.2,0.6,0.0,"T",1,0,1)
        wrttext("","Set = "+self.Ts,0.0,0.6,0.0,"T",1,0,1)
        wrttext("","Frame = "+str(self.row)+" of "+str(self.N-1),1.2,0.6,0.30,"T",1,0,1)
        if self.redshift[1]: wrttext("","z = %.4f" % (self.z),0.0,0.6,0.30,"T",1,0,1)
        wrttext("","Bin = "+str(self.bin),1.2,0.6,0.55,"T",1,0,1)
        wrttext("","Status = "+self.S,1.2,0.6,0.8,"T",1,0,1)
        wrttext("","Template = "+str(self.T),-1.2,0.6,0.0,"T",1,0,1)
        wrttext("","Mask = "+str(self.m),-2.4,0.6,0.0,"T",1,0,1)

    def DisplayStretch(self):
        ngs=256.; ch=1; itf=0; color=0
        cpgsch(ch/2.)
        cpgsvp( 0.10,0.90,0.85,0.875)
        cpgsci(0)
        z1=oldgray[0]; z2=oldgray[1]; gd=0
        if oldgray[1] < oldgray[0]: z2=oldgray[0]; z1=oldgray[1]; gd=1
        if oldgray[1] != oldgray[0]:
            cpgswin(z1,z2,0,1)
            cpgbox('BCMV',0,0,'BC',0,0)
            cpgsvp( 0.10,0.90,0.85,0.875)
        cpgsci(1)
        z1=newgray[0]; z2=newgray[1]; gd=0
        if newgray[1] < newgray[0]: z2=newgray[0]; z1=newgray[1]; gd=1
        cpgswin(z1,z2,0,1)
        cpgbox('BCMV',0,0,'BC',0,0)
        if gd: gs=arrayrange(z1,z2,(z2-z1)/ngs,Float32)
        else: gs=arrayrange(z2,z1,(z1-z2)/ngs,Float32)
        gs=array(transpose(gs).tolist(),Float32)
        ts=[z2,0.0,(z1-z2)/ngs,-0.5,1.0,0.0]
        ts=array(transpose(ts).tolist(),Float32)
        cpgsvp( 0.10,0.90,0.85,0.875)
        cpgsitf(itf)
        cpggray(gs,1,len(gs),1,1,1,len(gs),z1,z2,ts)
    def DisplayKey(self):
        wrttext("",str(self.key),-40.0,0.6,0.0,"T",1,0,1)
        if type(self.key) is TupleType:

            drawln(self.wra,self.key[1]*ones(self.wra.shape),ls=1,ci=3)
            drawln(self.key[0]*ones(self.ira.shape),self.ira,ls=1,ci=3)
    def DisplayM(self):
        for m in self.m:
            xm = array([m[0],m[1],m[1],m[0],m[0]])
            ym = array([self.imin,self.imin,self.imax,self.imax,self.imin])
            drawbox(xm,ym,5,1,1,3)
            drawbox(xm,ym,5,1,1,2)
    def DisplayRedshift(self):
        col=[0,4,2,1]
        wlist = self.redshift[0]
        w = []; a = []; s = []; v = []
        for wl in wlist:
            w.append(float(wl[0])*(1+self.z))
            a.append(col[int(wl[1])])
            s.append(wl[2])
            if self.waves[0] < w[-1] < self.waves[-1]:
               specval = self.spec1d[searchsorted(self.waves,w[-1])]
               specval = 0.15*(self.ira[-1] - specval) + specval
               v.append(specval)
            else:
               v.append(-999)
        [drawln(w[j]*ones(self.ira.shape),self.ira,ls=4,ci=a[j]) for j in range(len(w))]
        drawptl(w,v,s,ch=0.7,ci=a,lw=1,ang=90,fj=0)
    def DisplayChipRegions(self):
        for m in self.mapl[0][self.row]:
            c,x1,x2 = m
            xm = array([x1,x2,x2,x1,x1])
            ym = array([self.csecta,self.csecta,self.csectb,self.csectb,self.csecta])
            xp1 = (x1-self.crval1)/self.cdelt1
            xp2 = (x2-self.crval1)/self.cdelt1
            wp1 = (self.w1-self.crval1)/self.cdelt1
            wp2 = (self.w2-self.crval1)/self.cdelt1
            wscale = wp2-wp1
            if x1 >= self.w1 and x2 <= self.w2: xpp = (((xp2-xp1)/2.0)+xp1-wp1)/wscale
            elif x1 < self.w1 and x2 > self.w2: xpp = (((wp2-wp1)/2.0))/wscale
            elif x1 >= self.w1 and x2 > self.w2: xpp = (((wp2-xp1)/2.0)+xp1-wp1)/wscale
            elif x1 < self.w1 and x2 <= self.w2: xpp = (((xp2-wp1)/2.0))/wscale
            if xpp > 1.0 or xpp < 0: xpp = -10
            ypp = 2
            #print xm,ym
            drawbox(xm,ym,5,1,1,2)
            wrttext("","Chip %s" % (c),dist=-ypp,ch=0.6,len=xpp,along="T",lw=2,fj=0.5,ci=5)

### Arguements
argv,spec1d = rdarg(argv,"-1d",None,None)
argv,spec2d = rdarg(argv,"-big",None,None)
argv,dict = rdarg(argv,"-dict",None,"redshift.dict")
argv,direct = rdarg(argv,"-d",None,"redshiftCC")
argv,a1 = rdarg(argv,"-z1",float,-100)
argv,a2 = rdarg(argv,"-z2",float,100)
argv,chelp = rdarg(argv,"-h",None,0,single=1)
argv,w1 = rdarg(argv,"-w1",float,0)
argv,w2 = rdarg(argv,"-w2",float,0)
argv,imin = rdarg(argv,"-i1",float,0)
argv,imax = rdarg(argv,"-i2",float,0)
argv,fr = rdarg(argv,"-fr",int,0)
argv,trace = rdarg(argv,"-t",None,None)
argv,pgdev = rdarg(argv,"-p",None,"/xs")

argv,mapf = rdarg(argv,"-map",None,None)
argv,wavefile = rdarg(sys.argv,"-l",None,"/data1/mifune/kelson/Python.Linux/Basics/galaxylines.dat")
argv,watch = rdarg(argv,"-watch",None,0,single=1)
a10,a20 = 1*a1,1*a2
w10,w20 = 1*w1,1*w2
imin0,imax0 = 1*imin,1*imax

### Help
if chelp:
    cmd_help()
    sys.exit()

### Priming values
size = 9
opendev(pgdev,nx=1,ny=1,size=size,land=1,ask=0)

FITS1 = FITS(spec1d)
FITS2 = FITS(spec2d)
N = FITS1["naxis2"]
k="0"
xc=0
yc=0
zf = 2.0
key = None
mp1 = 1
mp2 = 1
cmp = 1
smo = {}
nsmo = {}
tsmo = 0
frz = 0 

TemplateIDs = {}
TemplateIDs["galaxy1"] = [23,27]
TemplateIDs["galaxy2"] = [23]
TemplateIDs["galaxy3"] = [27]
TemplateIDs["galaxy4"] = [24,25,26,27]
TemplateIDs["galaxy5"] = [23,24,25,26,27,28]
TemplateIDs["agn"] = [29,30,31,32]
TemplateIDs["star"] = [4,10,12]

### edataD: Add extra data associated with the frame here.
edataD = {}

tfun = None
if trace: tfun = pickle.load(open(trace))
edataD["trace"] = {"data":tfun,"status":0} 

mapL = []
if mapf:
    M_all = open(mapf).readlines()
    M = [m.split() for m in M_all]
    Sref = 0
    chipR = 0
    for s,m in enumerate(M):
        if chipR == 0: chipL = []
        if not m: continue
        elif m[0] == "SLIT":
            Sref = s 
            if chipR == 1: 
                mapL.append(chipL)
                chipR = 0
        elif m[0] == "CHIP": 
            chipR = 1
            chipL.append((m[1],float(m[4]),float(m[5])))
edataD["map"] = {"data":mapL,"status":0} 

if wavefile and os.path.exists(wavefile):
  wlines = open(wavefile,"r").readlines()
  wlines = filter(lambda l: l[0] != "#", wlines)
  wlist = map( split, wlines)
else:
  wlist = None
edataD["redshift"] = {"data":wlist,"status":0} 

### objD: Add data to be saved for each object here.
TemplateDict = {"ttype":"galaxy","tplate":TemplateIDs["galaxy1"],"status":"run","mask":[],"bin":"normal","z":0.0,"comments":None}
objD = {}
if dict and os.path.exists(dict):
    objD = eval(open(dict).read())
    for l in range(N): 
        for td in TemplateDict:
            if not objD[l].has_key(td): objD[l][td] = TemplateDict[td]
else:
    for l in range(N): objD[l] = {"object":FITS1["obj%03d" % (l+1)],"ttype":"galaxy","tplate":TemplateIDs["galaxy1"],"status":"run","mask":[],"bin":"normal","z":0.0,"comments":None}

### Start display tool
ngray=256
oldgray=[-999,999]
newobj = True

numkeys = ("0","1","2","3","4","5","6","7","8","9")
letterkeys = ("B","b","C","c","d","e","f","G","h","I","j","k","L","l","M","m","N","n","O","o","p","q","R","r","S","s","T","t","W","w","X","x","Y","y","Z","z")
charkeys = ("~","!","@","#","$","%","^","&","*","(",")","{","[","}","]",":",";","'","<",",",">",".","?","/"," ")
keys = numkeys + letterkeys + charkeys
 
Dnumkeys = ("0","1","2","3","4","5","6","7","8","9")
Dletterkeys = ("B","b","c","d","e","f","G","h","I","j","k","L","l","M","m","N","n","O","o","p","q","R","r","S","s","T","t","W","X","x","Y","y","Z","z")
Dcharkeys = ("!","@","#","$","%","^","&",")","{","[","}","]",":",";","'",",",".","?","/"," ")
Dkeys = Dnumkeys + Dletterkeys + Dcharkeys 
while k != "q":
    newgray=[a1,a2]
    if k in Dkeys:
        if newobj: Obj = SingleObject(fr,N,FITS1,FITS2)
        Obj.Display(a1,a2,w1,w2,imin,imax,objD[fr],edataD,key,smo)
    w1,w2,imin,imax,waves,name = Obj.GetInfo()
    newobj = False
    key = None

    status,xc,yc,k=cpgcurs(xc,yc)
    #print xc,yc,k
    xran = (w2 - w1)
    yran = (imax - imin)
    aran = a2-a1
    oldray=[a1,a2]
    if k in numkeys:
        if nsmo.keys() and tsmo: smo = {nsmo.keys()[0]:int(k)}; tsmo = 0
        else:
            if k == "1": objD[fr]["tplate"] = TemplateIDs["galaxy1"]; objD[fr]["ttype"] = "galaxy"
            if k == "2": objD[fr]["tplate"] = TemplateIDs["galaxy2"]; objD[fr]["ttype"] = "galaxy"
            if k == "3": objD[fr]["tplate"] = TemplateIDs["galaxy3"]; objD[fr]["ttype"] = "galaxy"
            if k == "4": objD[fr]["tplate"] = TemplateIDs["galaxy4"]; objD[fr]["ttype"] = "galaxy"
            if k == "5": objD[fr]["tplate"] = TemplateIDs["galaxy5"]; objD[fr]["ttype"] = "galaxy"
            if k == "6": objD[fr]["tplate"] = TemplateIDs["agn"]; objD[fr]["ttype"] = "agn"
            if k == "7": objD[fr]["tplate"] = TemplateIDs["star"]; objD[fr]["ttype"] = "star"

    if k == "<":
        size -= 2
        if size < 3: size = 3
        closedev()
        opendev(pgdev,nx=1,ny=1,size=size,land=1,ask=0)
    if k == ">":
        size += 2
        if size > 15: size = 15
        closedev()
        opendev(pgdev,nx=1,ny=1,size=size,land=1,ask=0)

    if k == "r": w1,w2,imin,imax,a1,a2 = w10,w20,imin0,imax0,a10,a20
    if k == "x" or k == "z":
        xran = xran*zf; w1 = xc - xran/2.; w2 = xc + xran/2.
        if w1 < waves[0]: w1 = waves[0]
        if w2 > waves[-1]: w2 = waves[-1]
    if k == "X" or k == "Z":
        xran = xran/zf; w1 = xc - xran/2.; w2 = xc + xran/2.
        if w1 < waves[0]: w1 = waves[0]
        if w2 > waves[-1]: w2 = waves[-1]
    if k == "y" or k == "z":
        yran = yran*zf
        imin = yc - yran/2.; imax = yc + yran/2.
    if k == "Y" or k == "Z":
        yran = yran/zf
        imin = yc - yran/2.; imax = yc + yran/2.
    if k == "h":
        w1 = w1 - xran/10.
        if w1 < waves[0]: w1 = waves[0]
        w2 = w1 + xran
    if k == "l":
        w2 = w2 + xran/10.
        if w2 > waves[-1]: w2 = waves[-1]
        w1 = w2 - xran
    if k == "j" or k == "k":
        if k =="j": direc = -1
        if k =="k": direc = +1
        imin = imin + direc*yran/10.; imax = imax + direc*yran/10.
    if k == "c":
        newobj = True
        maps = edataD["map"]["status"]
        if maps: maps = 0
        else: maps = 1
        edataD["map"]["status"] = maps
    if k == "s":
        stati = ["run","skip","done"] 
        statp = objD[fr]["status"]
        for l,stat in enumerate(stati): 
            if statp == stat:
                if l+1 <= len(stati)-1: objD[fr]["status"] = stati[l+1]
                else: objD[fr]["status"] = stati[0]
    if k == "S":
        f = open(dict,"w")
        f.write(str(objD))
        f.close()
        key = "File saved."
    if k == "e": 
        if objD[fr]["bin"]=="ebin": objD[fr]["bin"]="normal"
        elif objD[fr]["bin"]=="normal": objD[fr]["bin"]="ebin"
    if k == "d":
        if not mp1:
            choice = raw_input("Are you sure you want to delete all masks (default no)? ")
            if not choice: choice = "no"
            if choice.lower().strip()[0] == "y": 
                for l in range(N): objD[l]["mask"]=[]
            mp1 = 1
        elif not mp2:
            objD[fr]["mask"]=[]
            mp2 = 1
    if k == "T":
       if edataD["trace"]["status"]: edataD["trace"]["status"] = 0
       else: edataD["trace"]["status"] = 1
    if k == "t":
        if objD[fr]["tplate"]: print "\nPrevious templates: \n%s" % (objD[fr]["tplate"])
        temp = raw_input("Enter a list of new templates (CR unchanged)? ")
        if temp: objD[fr]["tplate"] = eval(temp)
    if k == "C":
        if objD[fr]["comments"]: print "\nPrevious comments: \n%s" % (objD[fr]["comments"])
        else: print
        temp = raw_input("Enter comments (CR unchanged)? ")
        if temp: objD[fr]["comments"] = temp
    if k == "M":
        if not mp1:
            mask = sort([c1,int(xc)])
            overlap = -1
            for mold in objD[fr]["mask"]:
               if mask[0] < mold[1] and mask[1] > mold[0]:
                 overlap = objD[fr]["mask"].index(mold)
            if overlap > -1:
              mold = objD[fr]["mask"][overlap]
            for l in range(N):
              if overlap > -1:
                loverlap = -1
                for lmold in objD[l]["mask"]:
                   if mold == lmold:
                      loverlap = objD[l]["mask"].index( lmold); break
                if loverlap > -1:
                   if mask[0] > mold[0] and mask[1] < mold[1]:
                     objD[l]["mask"][loverlap] = mask
                   else:
                     objD[l]["mask"][loverlap] = [min([mask[0],mold[0]]), max([mask[1],mold[1]])]
              else:
                objD[l]["mask"].append(mask)
            mp1 = 1
        elif mp1:
            c1 = int(xc)
            mp1 = 0
            key = "Press M one more time."
    if k == "m":
        if not mp2:
            mask = sort([c1,int(xc)])
            overlap = -1
            for mold in objD[fr]["mask"]:
               if mask[0] < mold[1] and mask[1] > mold[0]:
                 overlap = objD[fr]["mask"].index(mold)
            if overlap > -1:
              mold = objD[fr]["mask"][overlap]
              if mask[0] > mold[0] and mask[1] < mold[1]:
                objD[fr]["mask"][overlap] = mask
              else:
                objD[fr]["mask"][overlap] = [min([mask[0],mold[0]]), max([mask[1],mold[1]])]
            else:
              objD[fr]["mask"].append(mask)
            mp2 = 1
        elif mp2:
            c1 = int(xc)
            mp2 = 0
            key = "Press m one more time."

    ### Chip masking
    #if k == "":
    #    if not cmp:
    #        mask = sort([c1,int(xc)])
    #        for c,x1,x2 in edataD["map"]["data"][fr]:
    #            if mask[0] >= x1 and mask[1] <= x2: cx1 = (mask[0] - x1)/Obj.cdelt1; cx2 = (mask[1] - x1)/Obj.cdelt1

    #        #overlap = -1
    #        #for mold in objD[fr]["mask"]:
    #        #    if mask[0] < mold[1] and mask[1] > mold[0]:
    #        #        overlap = objD[fr]["mask"].index(mold)
    #        #if overlap > -1:
    #        #    mold = objD[fr]["mask"][overlap]
    #        print mask,x1,x2,cx1,cx2
    #        for l in range(N):
    #            mask = None
    #            for cl,lx1,lx2 in edataD["map"]["data"][l]:
    #                if c == cl:
    #                    print c,lx1,lx2,cx1,cx2
    #                    mask = [lx1+(cx1*Obj.cdelt1),lx1+(cx2*Obj.cdelt1)]
    #            if mask: 
    #          #      if overlap > -1:
    #          #          loverlap = -1
    #          #          for lmold in objD[l]["mask"]:
    #          #             if mold == lmold:
    #          #                loverlap = objD[l]["mask"].index( lmold); break
    #          #          if loverlap > -1:
    #          #             if mask[0] > mold[0] and mask[1] < mold[1]:
    #          #                 objD[l]["mask"][loverlap] = mask
    #          #             else:
    #          #                 objD[l]["mask"][loverlap] = [min([mask[0],mold[0]]), max([mask[1],mold[1]])]
    #          #      else:
    #                    print mask
    #                    objD[l]["mask"].append(mask)
    #        cmp = 1
    #    elif cmp:
    #        c1 = int(xc)
    #        cmp = 0
    #        key = "Press Ctrl+c one more time."

    if k == "N":
        nl = raw_input("Enter noise level to mask out (default 1e5)? ")
        if not nl: nl = 1e5
        nl = float(nl)
        width = raw_input("Enter width of mask (default 5)? ")
        if not width: width = 5.0
        width = float(width)
        if nl > 0: nparam = greater(Obj.noise1d,float(nl))
        if nl < 0: nparam = less(Obj.noise1d,float(nl))
        for n in compress(nparam,waves):
            mask = [int(n-width+0.5),int(n+width+0.5)]
            overlap = -1
            for mold in objD[fr]["mask"]:
                if mask[0] < mold[1] and mask[1] > mold[0]:
                    overlap =  objD[fr]["mask"].index(mold)
            if overlap > -1:
                mold = objD[fr]["mask"][overlap]
                if mask[0] > mold[0] and mask[1] < mold[1]:
                    objD[fr]["mask"][overlap] = mask
                else:
                    objD[fr]["mask"][overlap] = [min([mask[0],mold[0]]),max([mask[1],mold[1]])]
            else:
                objD[fr]["mask"].append(mask) 
    if k == "W":
        nl = raw_input("Enter spectra level to mask out (default -10)? ")
        if not nl: nl = -10
        nl = float(nl)
        width = raw_input("Enter width of mask (default 10)? ")
        if not width: width = 10.0
        width = float(width)
        nparam = less(Obj.spec1d,float(nl))
        for n in compress(nparam,waves):
            mask = [int(n-width+0.5),int(n+width+0.5)]
            overlap = -1
            for mold in objD[fr]["mask"]:
                if mask[0] < mold[1] and mask[1] > mold[0]:
                    overlap =  objD[fr]["mask"].index(mold)
            if overlap > -1:
                mold = objD[fr]["mask"][overlap]
                if mask[0] > mold[0] and mask[1] < mold[1]:
                    objD[fr]["mask"][overlap] = mask
                else:
                    objD[fr]["mask"][overlap] = [min([mask[0],mold[0]]),max([mask[1],mold[1]])]
            else:
                objD[fr]["mask"].append(mask) 
    if k == "o":
        nl = 0.0
        width = raw_input("Enter width of mask (default 5)? ")
        if not width: width = 5.0
        width = float(width)
        nparam = equal(Obj.spec1d,float(nl))
        for n in compress(nparam,waves):
            mask = [int(n-width+0.5),int(n+width+0.5)]
            overlap = -1
            for mold in objD[fr]["mask"]:
                if mask[0] < mold[1] and mask[1] > mold[0]:
                    overlap =  objD[fr]["mask"].index(mold)
            if overlap > -1:
                mold = objD[fr]["mask"][overlap]
                if mask[0] > mold[0] and mask[1] < mold[1]:
                    objD[fr]["mask"][overlap] = mask
                else:
                    objD[fr]["mask"][overlap] = [min([mask[0],mold[0]]),max([mask[1],mold[1]])]
            else:
                objD[fr]["mask"].append(mask) 
    if k == "O":
        choice = raw_input("Are you sure you want to mask all spectra identical to zero (default no)? ")
        if not choice: choice = "no"
        if choice.lower().strip()[0] == "y": 
            nl = 0.0
            width = raw_input("Enter width of mask (default 5)? ")
            if not width: width = 5.0
            width = float(width)
            for l in range(N):
                nparam = equal(GetSpectra1D(l,FITS1)[0],float(nl))
                for n in compress(nparam,waves):
                    mask = [int(n-width+0.5),int(n+width+0.5)]
                    overlap = -1
                    for mold in objD[l]["mask"]:
                        if mask[0] < mold[1] and mask[1] > mold[0]:
                            overlap =  objD[l]["mask"].index(mold)
                    if overlap > -1:
                        mold = objD[l]["mask"][overlap]
                        if mask[0] > mold[0] and mask[1] < mold[1]:
                            objD[l]["mask"][overlap] = mask
                        else:
                            objD[l]["mask"][overlap] = [min([mask[0],mold[0]]),max([mask[1],mold[1]])]
                    else:
                        objD[l]["mask"].append(mask) 


    frold = fr

    if k == "L":
        f = open(dict,"w")
        f.write(str(objD))
        f.close()
        FITS1.close()
        FITS2.close()
        FITS1 = FITS(spec1d)
        FITS2 = FITS(spec2d)
        if trace: tfun = pickle.load(open(trace))
        edataD["trace"]["data"] = tfun 
        N = FITS1["naxis2"]
        if dict and os.path.exists(dict):
            objD = eval(open(dict).read())
            for l in range(N): 
                for td in TemplateDict:
                    if not objD[l].has_key(td): objD[l][td] = TemplateDict[td]
        else:
            for l in range(N): objD[l] = {"object":FITS1["obj%03d" % (l+1)],"ttype":"galaxy","tplate":TemplateIDs["galaxy1"],"status":"run","mask":[],"bin":"normal","z":0.0}
        newobj = True
    # freeze wavelengths, intensity, stretch
    if k == "F": frz = 1
    if k == "b":
        fr=0
        if not frz: w1,w2,imin,imax,a1,a2 = w10,w20,imin0,imax0,a10,a20
    if k == "f":
        fr=N-1
        if not frz: w1,w2,imin,imax,a1,a2 = w10,w20,imin0,imax0,a10,a20
    if k == "n":
        if fr!=N-1: fr+=1
        if not frz: w1,w2,imin,imax,a1,a2 = w10,w20,imin0,imax0,a10,a20
    if k == "p":
        if fr!=0: fr-=1
        if not frz: w1,w2,imin,imax,a1,a2 = w10,w20,imin0,imax0,a10,a20
    if frold != fr:
        newobj = True

    if k == ";": a1=a1+aran/4.; a2=a2-aran/4.
    if k == "'": a1=a1-aran/4.; a2=a2+aran/4.
    if k == ",": a1=a1+aran/4.; a2=a2+aran/4.
    if k == ".": a1=a1-aran/4.; a2=a2-aran/4.
    if k == "/": a1,a2=a2,a1

    if k == "w": watch = 1
    if k == "V":
        fname = "%04d_%s.dict" % (fr,name)
        f = open(fname,"w")
        f.write(str({fr:objD[fr]}))
        f.close()
        command = " ".join(["aRedshift.py",spec1d,"-nz","100","-imacs","-input",fname,["","-watch"][watch],"-d",direct,"-single"])
        pid=os.fork()
        #print command
        if pid==0:
            os.system(command)
            os._exit(0)
        else:
            os.waitpid(pid,0)
            os.remove(fname)
            print "---- Redshift Computed ----"

    if k == "!": objD[fr]["z"] = 0.00
    if k == "@": objD[fr]["z"] = 0.10
    if k == "#": objD[fr]["z"] = 0.35 
    if k == "$": objD[fr]["z"] = 0.75
    if k == "%": objD[fr]["z"] = 1.50
    if k == "^": objD[fr]["z"] = 3.00
    if k == "&": objD[fr]["z"] = 5.00
    #if k == "*": objD[fr]["z"] = 
    #if k == "(": objD[fr]["z"] =
    if k == ")":
        redp = edataD["redshift"]["status"]
        if redp: redp = 0
        else: redp = 1
        edataD["redshift"]["status"] = redp

    if k in ["[","]","{","}"] and wlist and edataD["redshift"]["status"]:
        zt = objD[fr]["z"]
        if k == "[": zt = zt - 0.01
        if k == "]": zt = zt + 0.01
        if k == "{": zt = zt - 0.0001
        if k == "}": zt = zt + 0.0001
        objD[fr]["z"] = zt

    if k == "~":
        print "\n  %3s %12s  %6s %9s %6s  %s" % ("row","object","z","templates","status","comments")
        print "-"*43
        for l in range(N):
            if objD[l]["comments"]: comments = objD[l]["comments"]
            else: comments = ""
            print "%5i %12s  %.4f  %8s %6s  %s"  % (l,objD[l]["object"].strip(),objD[l]["z"],objD[l]["tplate"],objD[l]["status"],comments)
        print


    
    if k == "R":
        manred = raw_input("Enter redshift? ")
        if not manred: manred = objD[fr]["z"]
        objD[fr]["z"] = float(manred)

    if k == "B" or k == "G":
        if smo:
            if smo.keys()[0] != k: nsmo = {k:0}
        else:
            if k == "B": nsmo = {"B":0}
            if k == "G": nsmo = {"G":0}
        key = "Press a number."
        tsmo = 1
 
    if k == " ":
       key = xc,yc

    if k == ":":
        try: exec(raw_input("<cmd?> "))
        except: pass

    if k == "?":
        khelp()

    if k == "I":
        raise "User Interrupt"

f = open(dict,"w")
f.write(str(objD))
f.close()

FITS1.close()
FITS2.close()

closedev()
