#!/usr/bin/env python

import pgplot as PG
import sys
from MLab import diff
from FITS import *
from basics import *
from Multipack import *

def plotspec(w,object,fitsfile,galaxywaves,galaxyids,galaxyn,lines,label):
    irange =  extrema(object)
    PG.newbox([w[0],w[len(w)-1]],[irange[0],1.5*irange[1]],ch=1.5)
    if title and k is 0: PG.wrttext("",title,1.0,1.4,0.0,"T",1,0,1)
    PG.drawln(w,object)
    PG.title(label,ch=1.5)
    #print w[0],w[len(w)-1]

    if lines:
        extr = extrema(object)
        ran = diff(extr)[0]
        ymar = extr[0]-0.05*ran, extr[1]+0.70*ran
        tck = splrep(w,object,s=0,k=3)
        galaxyl = splev(galaxywaves,tck)
        galaxyh = (ymar[1]-0.05*ran)*ones((galaxyn,))
        galaxyp = 0.25*(galaxyh-galaxyl) + galaxyl
        PG.markpts(w[0],w[len(w)-1],ymar[0],ymar[1],galaxywaves,galaxyp,galaxyids,ch=0.7,ci=1,ang=90,fj=0,lw=1)
        [PG.drawln([galaxywaves[j],galaxywaves[j]],[galaxyl[j],galaxyp[j]],ls=4) for j in range(galaxyn)]

    PG.xlabel("wavelength (\A)",ch=1.5)
    PG.ylabel("counts",ch=1.5)
    PG.wrttext("",fitsfile,0.5,1.5,0.85,"T",1,0,1)

pre = "spDR2-"
ext = "fit"
dig = "3"
num = 33
name = "SDSS-Templates"
title = "SDSS - Spectral cross-correlation templates"
galaxylines = "/data1/hidari/gwalth/Cosmos/linelists/galaxylines.dat"
galaxylabels = "labels.txt"
lines = False
fran = None

for k,arg in enumerate(sys.argv):
    if arg in ("-r","--range"): fran = sys.argv[k+1].split("-")
    if arg in ("-n","--name"): name = sys.argv[k+1]
    if arg in ("-t","--title"): title = sys.argv[k+1]
    if arg in ("-g","--galaxylines"): galaxylines = sys.argv[k+1]
    if arg in ("-l","--lines"): lines = True

if lines: PG.opendev("%s.ps/cps" % (name),nx=1,ny=2,land=0,size=10,asp=0.8)
else: PG.opendev("%s.ps/cps" % (name),nx=2,ny=4,land=0,size=10,asp=0.8)

if fran: ran = range(int(fran[0]),int(fran[1])+1)
else: ran = range(num)

labels = [item.strip() for item in open(galaxylabels,"r")]
for k in ran:
    fitstr = "".join(["%s%0",dig,"d.",ext])
    fitsfile = fitstr % (pre,k)
    f = FITS(fitsfile)
    object = f.data()[0]
    w = (arange(f["naxis1"])+1.0-f["crpix1"])*f["cdelt1"] + f["crval1"]
    if f["dc-flag"] is 1: w = power(10,w)


    galaxylinesdata = [item.split() for item in open(galaxylines,"r") if item[0] != "#"]
    galaxywaves = [float(galaxyline[0]) for galaxyline in galaxylinesdata]
    galaxyids = [" ".join([galaxyline[0]]+galaxyline[1:]) for galaxyline in galaxylinesdata]
    galaxyn = len(galaxyids)

    plotspec(w,object,fitsfile,galaxywaves,galaxyids,galaxyn,lines,labels[k])
    f.close()
PG.closedev()
